/*
 * Copyright (C) 2015 MediaTek Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */

/*****************************************************************************
 *
 * Filename:
 * ---------
 *   nt99141yuv_Sensor.c
 *
 * Project:
 * --------
 *   MAUI
 *
 * Description:
 * ------------
 *   Image sensor driver function
 *   V1.0.0
 *
 * Author:
 * -------
 *   Mormo
 *
 *=============================================================
 *             HISTORY
 * Below this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *------------------------------------------------------------------------------
 * $Log$
 * 2011/11/03 Firsty Released By Mormo;
 *   
 *
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *=============================================================
 ******************************************************************************/
#include <linux/videodev2.h>
#include <linux/i2c.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/cdev.h>
#include <linux/uaccess.h>
#include <linux/fs.h>
#include <asm/atomic.h>
#include "kd_camera_hw.h"
#include "kd_imgsensor.h"
#include "kd_imgsensor_define.h"
#include "kd_imgsensor_errcode.h"
#include "kd_camera_feature.h"
#include "nt99141yuv_Sensor.h"
#include "nt99141yuv_RegVersion.h"
#include "nt99141yuv_Camera_Sensor_para.h"
#include "nt99141yuv_CameraCustomized.h"
/*******************************************************************************
 * // Adapter for Winmo typedef
 ********************************************************************************/
#define WINMO_USE 0

#define Sleep(ms) mdelay(ms)
#define RETAILMSG(x,...)
#define TEXT

#define PFX         "[NT99141YUV]: "
#define DEBUG	1
#if DEBUG
    #define LOG_DEBUG(format, args...)	pr_debug( PFX "[%s] " format, __FUNCTION__, ##args)
#else
    #define LOG_DEBUG(format, args...)   
#endif


#define PREVIEW_USE_VGA     0
#define PREVIEW_USE_SVGA    0 // 1
#define PREVIEW_USE_720P    1 // 0

#if PREVIEW_USE_VGA
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_VGA_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_VGA_HEIGHT - 6)
#endif

#if PREVIEW_USE_SVGA
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_SVGA_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_SVGA_HEIGHT - 6)
#endif

#if PREVIEW_USE_720P
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_720P_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_720P_HEIGHT - 6)
#endif

kal_bool   NT99141_MPEG4_encode_mode = KAL_FALSE;
kal_uint16 NT99141_dummy_pixels = 0, NT99141_dummy_lines = 0;
kal_bool   NT99141_MODE_CAPTURE = KAL_FALSE;

static kal_int8 NT99141_DELAY_AFTER_PREVIEW = -1;

kal_bool   NT99141_CAM_BANDING_50HZ = KAL_TRUE;//lj_test KAL_FALSE;
kal_bool   NT99141_CAM_Nightmode = 0;
kal_bool	setshutter = KAL_FALSE;

kal_uint32 g_imgsensor_fps = 300;
kal_uint32 NT99141_isp_master_clock;
kal_bool   fromsd = false;

static kal_uint32 NT99141_g_fPV_PCLK = 24;
kal_uint8 NT99141_sensor_write_I2C_address = NT99141_WRITE_ID;
kal_uint8 NT99141_sensor_read_I2C_address = NT99141_READ_ID;

UINT8 NT99141PixelClockDivider=0;

MSDK_SENSOR_CONFIG_STRUCT NT99141SensorConfigData;


#define NT99141YUV_DEBUG
#ifdef NT99141YUV_DEBUG
#define SENSORDB(fmt, args...)  pr_info(PFX "%s: " fmt, __FUNCTION__ ,##args)
#else
#define SENSORDB(x,...)
#endif
extern int iReadRegI2C(u8 *a_pSendData , u16 a_sizeSendData, u8 * a_pRecvData, u16 a_sizeRecvData, u16 i2cId);
extern int iWriteRegI2C(u8 *a_pSendData , u16 a_sizeSendData, u16 i2cId);


/* Sensor output window information */
static SENSOR_WINSIZE_INFO_STRUCT imgsensor_winsize_info[5]=
{{ 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  8,    6, 1264,  708},  // Preview
 { 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  8,    6, 1264,  708}, // capture
 { 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  8,    6, 1264,  708},  // video
 { 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  160,    240, 1280,  720}, //hight speed video
 { 1280, 720,      0,    0, 1280, 720, 1280,  720,      0,   0, 1280,  720,  160,    240, 1280,  720}}; // slim video

void NT99141_write_cmos_sensor(kal_uint16 addr, kal_uint8 para)
{
	char puSendCmd[4] = {(char)((addr >> 8) & 0xFF) ,(char)(addr & 0xFF), (char)(para & 0xFF)};
	kdSetI2CSpeed(400);
	iWriteRegI2C(puSendCmd , 3, NT99141_WRITE_ID);
}
kal_uint8 NT99141_read_cmos_sensor(kal_uint16 addr)
{
	kal_uint8 get_byte=0;
	char puSendCmd[2] = { (char)((addr >> 8) & 0xFF) ,(char)((addr  >> 0)& 0xFF) };

	kdSetI2CSpeed(400);
	iReadRegI2C(puSendCmd , 2, (u8*)&get_byte, 1, NT99141_WRITE_ID);
	
	return get_byte;
}

/*************************************************************************
 * FUNCTION
 *	NT99141_SetShutter
 *
 * DESCRIPTION
 *	This function set e-shutter of NT99141 to change exposure time.
 *
 * PARAMETERS
 *   iShutter : exposured lines
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99141_Set_Shutter(kal_uint16 iShutter)
{
	SENSORDB("line %d\r\n",__LINE__);
} /* Set_NT99141_Shutter */


/*************************************************************************
 * FUNCTION
 *	NT99141_read_Shutter
 *
 * DESCRIPTION
 *	This function read e-shutter of NT99141 .
 *
 * PARAMETERS
 *  None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
kal_uint16 NT99141_Read_Shutter(void)
{
	kal_uint16 shutter = 0;
	SENSORDB("line %d\r\n",__LINE__);
	return shutter;
} /* NT99141_read_shutter */


/*************************************************************************
 * FUNCTION
 *	NT99141_config_window
 *
 * DESCRIPTION
 *	This function config the hardware window of NT99141 for getting specified
 *  data of that window.
 *
 * PARAMETERS
 *	start_x : start column of the interested window
 *  start_y : start row of the interested window
 *  width  : column widht of the itnerested window
 *  height : row depth of the itnerested window
 *
 * RETURNS
 *	the data that read from NT99141
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
//void NT99141_config_window(kal_uint16 startx, kal_uint16 starty, kal_uint16 width, kal_uint16 height)//Ronlus
void NT99141_config_window(void)
{
	int fps = 0;
	struct reglist *list = NULL;
	fps = g_imgsensor_fps  / 10 ;

	SENSORDB("line %d  SET FPS %d\r\n",__LINE__,fps); 
	
#ifdef NT99141_SENSOR_DEBUG
	if(fromsd)
		list = nt99141_preview;
#endif 
	
	//fps = 15;

	if(fps == 5){
		list = preview_24M_720P_5fps;
	}
	else if (fps == 15){
		list = preview_24M_720P_15fps;
	}
	else if (fps == 25){
		list = preview_24M_720P_25fps;
	}
	else { //default 30fps
		list = preview_24M_720P_30fps;
	}						 
		
	while(list->addr != 0xffff)
	{
		NT99141_write_cmos_sensor(list->addr,list->val);
		list++;
	}

	
} /* NT99141_config_window */



/*************************************************************************
 * FUNCTION
 *	NT99141_SetGain
 *
 * DESCRIPTION
 *	This function is to set global gain to sensor.
 *
 * PARAMETERS
 *   iGain : sensor global gain(base: 0x40)
 *
 * RETURNS
 *	the actually gain set to sensor.
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
kal_uint16 NT99141_SetGain(kal_uint16 iGain)
{
	SENSORDB("line %d\r\n",__LINE__);
	return iGain;
}


/*************************************************************************
 * FUNCTION
 *	NT99141_NightMode
 *
 * DESCRIPTION
 *	This function night mode of NT99141.
 *
 * PARAMETERS
 *	bEnable: KAL_TRUE -> enable night mode, otherwise, disable night mode
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99141_night_mode(kal_bool bEnable)
{
	SENSORDB("line %d\r\n",__LINE__);
} 

/*************************************************************************
 * FUNCTION
 *	NT99141_Write_More_Registers
 *
 * DESCRIPTION
 *	This function is served for FAE to modify the necessary Init Regs. Do not modify the regs
 *     in init_NT99141() directly.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99141_Write_More_Registers(void)
{
	SENSORDB("line %d\r\n",__LINE__);
}

/*************************************************************************
 * FUNCTION
 *	NT99141_Sensor_Init
 *
 * DESCRIPTION
 *	This function apply all of the initial setting to sensor.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 *************************************************************************/

void NT99141_Sensor_Init(void)
{
	struct reglist *list=NULL;
	SENSORDB("line %d\r\n",__LINE__);
#ifdef INIT_TRULY_PN17034_20180306_ANNIEFU
	list = init_truly_pn17034_20180306_AnnieFu;
//#elif INIT_XXX_PN123_2000000000_XXXXX
//	list = ;
#else //
	list = init_truly_pn17034_20180308_from3275;
#endif 

	while(list->addr != 0xffff)
	{
		NT99141_write_cmos_sensor(list->addr,list->val);
		list++;
	}

} 


u8 NT99141_Initialize_from_T_Flash(void)
{

#ifdef NT99141_SENSOR_DEBUG

	u8 *curr_ptr = NULL;
	u32 file_size = 0;
	struct file *fp; 
	mm_segment_t fs; 
	loff_t pos = 0; 
	unsigned int index,regaddr, regval;
	static u8 data_buff[10*1024] ;

    fp = filp_open("/data/camsensor/nt99141_sd.txt", O_RDONLY , 0); 
    if (IS_ERR(fp)) { 
		SENSORDB("create file error 0\n");
		fp = filp_open("/data/camsensor/nt99141_sd.txt", O_RDONLY , 0); 
    	if (IS_ERR(fp)) { 
	        SENSORDB("create file error 1\n"); 
	        return -1;
    	}
    } 
    else
		SENSORDB("NT99141_Initialize_from_T_Flash Open File Success\n");
	
	fs = get_fs(); 
	set_fs(KERNEL_DS); 

	file_size = vfs_llseek(fp, 0, SEEK_END);
	vfs_read(fp, data_buff, file_size, &pos); 
	filp_close(fp, NULL); 
	set_fs(fs);


	/* Start parse the setting witch read from t-flash. */
	curr_ptr = data_buff;
	while (curr_ptr < (data_buff + file_size))
	{

	
		//SENSORDB("debug  char in %c\n",*curr_ptr);
		while ((*curr_ptr == ' ') || (*curr_ptr == '\t'))/* Skip the Space & TAB */
			curr_ptr++;				

		if (((*curr_ptr) == '/') && ((*(curr_ptr + 1)) == '*'))
		{
			if(!strncmp((const char *)curr_ptr,"/*Format",8)){
				SENSORDB("Find Format Section\n");
				while (!(((*curr_ptr) == '*') && ((*(curr_ptr + 1)) == '/')))
				curr_ptr++;		/* Skip block comment code. */														
			}

			else if(!strncmp((const char *)curr_ptr,"/*Version*/",11)){
				char version[128];
				SENSORDB("Camera sensor nt99141 version:\n");

				curr_ptr += 11;

				if((*curr_ptr) == '{')
					sscanf(curr_ptr,"{%s}",(char *)version);
				else 
					curr_ptr++;
		
				while((*curr_ptr) != '}'){
					curr_ptr++;
				}
				
				SENSORDB("%s",version);
			}

			
			else if(!strncmp((const char *)curr_ptr,"/*Init*/",8)){
				index = 0;
				curr_ptr += 8;
				SENSORDB("Find Init Section\n");
				while(1)
				{

					if (((*curr_ptr) == '/') && ((*(curr_ptr + 1)) == '/')){
						SENSORDB("Skip annotation parameters\n ");
						curr_ptr += 3;
					}

					if(index > REG_INIT_COUNT){
						SENSORDB("Parsing parameters error,to many register list\n ");
						return -2;
					}

					
					if(!strncmp((const char *)curr_ptr,"reg",3)){
						sscanf(curr_ptr,"reg:%x=%x",&regaddr,&regval);
						//SENSORDB("Parsing reg addr %x val %x\n",regaddr,regval);
						curr_ptr += 15;
						nt99141_init[index].addr = regaddr;
						nt99141_init[index].val = regval;

						if(regaddr == 0xffff && regval == 0xff)
							break;

						index++;
					}
					else 
						curr_ptr++;
					
				}
			}

			else if(!strncmp ((const char *)curr_ptr,"/*Preview*/",11)){
				index = 0;
				curr_ptr += 11;
				SENSORDB("Find Preview Section\n");
				while(1)
				{
					if (((*curr_ptr) == '/') && ((*(curr_ptr + 1)) == '/')){
						SENSORDB("Skip annotation parameters\n ");
						curr_ptr += 3;
					}

					if(index > REG_PREVIEW_COUNT){
						SENSORDB("Parsing parameters error,to many register list\n ");
						return -2;
					}
					if(!strncmp((const char *)curr_ptr,"reg",3)){
						sscanf(curr_ptr,"reg:%x=%x",&regaddr,&regval);
						//SENSORDB("Parsing reg addr %x val %x\n",regaddr,regval);
						curr_ptr += 15;
						nt99141_preview[index].addr = regaddr;
						nt99141_preview[index].val = regval;

						if(regaddr == 0xffff && regval == 0xff)
							break;
						
						index++;
					}
					else 
						curr_ptr++;

				}

			}

			else if(!strncmp ((const char *)curr_ptr,"/*End*/",7)){
				SENSORDB("Find End Section\n");
				break;
			}

			else {
				SENSORDB("Prase Register data error\n");
				return -2;
			}
			
		}
		
		curr_ptr++; 

	}

	/*Set Init*/

	index = 0;
	do{
		//SENSORDB("Write register 0x%x:0x%x\n",nt99141_init[index].addr,nt99141_init[index].val);
		NT99141_write_cmos_sensor(nt99141_init[index].addr,nt99141_init[index].val);
		index++;
	}while(nt99141_init[index].addr != 0xffff);

#if 0 // move to NT99141_config_window
	/*Set Preview*/
	index = 0;
	do{
		//SENSORDB("Write register 0x%x:0x%x\n",nt99141_preview[index].addr,nt99141_preview[index].val);
		NT99141_write_cmos_sensor(nt99141_preview[index].addr,nt99141_preview[index].val);
		index++;
	}while(nt99141_preview[index].addr != 0xffff);
#endif 

	SENSORDB("Seting sensor Register from SdCard Sucessfully!\n");

	fromsd = true;
	return 0;	

#endif 

	return -1;
}


/*************************************************************************
 * FUNCTION
 *	NT99141Open
 *
 * DESCRIPTION
 *	This function initialize the registers of CMOS sensor
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99141Open(void)
{
	int i;
	kal_uint16 sensor_id=0;
	
	SENSORDB("line %d\r\n",__LINE__);
	// check if sensor ID correct

	for(i = 0; i < 3; i++)
	{
		sensor_id = NT99141_read_cmos_sensor(0x3000);
		sensor_id = (sensor_id << 8) | NT99141_read_cmos_sensor(0x3001);
		if (sensor_id == NT99141_YUV_SENSOR_ID)
		{
			break;
		}
	}
	
	if(sensor_id != NT99141_YUV_SENSOR_ID)
	{
		SENSORDB("@@@SP:NT99141 Sensor id read failed, ID = %x\n", sensor_id);
		return ERROR_SENSOR_CONNECT_FAIL;
	}

	SENSORDB("NT99141 Sensor id read, ID = %x\n", sensor_id);

	//fp = filp_open("/data/camsensor/nt99141_sd.txt", O_RDONLY , 0); 
	//if (!IS_ERR(fp)){ 
	//	SENSORDB("open sensor data file success\n");
	//	filp_close(fp, NULL); 
	//}

	if(NT99141_Initialize_from_T_Flash()){
		NT99141_Sensor_Init();
		NT99141_Write_More_Registers();//added for FAE to debug
	}
		
	return ERROR_NONE;
} /* NT99141Open */


/*************************************************************************
 * FUNCTION
 *	NT99141Close
 *
 * DESCRIPTION
 *	This function is to turn off sensor module power.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99141Close(void)
{
	SENSORDB("line %d\r\n",__LINE__);
	return ERROR_NONE;
} /* NT99141Close */

void NT99141_AfterSnapshot(void)
{
	SENSORDB("line %d\r\n",__LINE__);
}  


void NT99141_BeforeSnapshot(void)
{
	SENSORDB("line %d\r\n",__LINE__);
} 

/*************************************************************************
 * FUNCTION
 * NT99141Preview
 *
 * DESCRIPTION
 *	This function start the sensor preview.
 *
 * PARAMETERS
 *	*image_window : address pointer of pixel numbers in one period of HSYNC
 *  *sensor_config_data : address pointer of line numbers in one period of VSYNC
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99141Preview(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
		MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	
	SENSORDB("line %d\r\n",__LINE__);
    NT99141_MODE_CAPTURE = KAL_FALSE;
    NT99141_config_window();
	
    if (setshutter) 
    {
	    ///NT99141_AfterSnapshot(); 
    }
	setshutter = KAL_FALSE;
	if(sensor_config_data->SensorOperationMode == MSDK_SENSOR_OPERATION_MODE_VIDEO)		// MPEG4 Encode Mode
	{
		SENSORDB("@@@SP:Ronlus video preview\r\n");
		NT99141_MPEG4_encode_mode = KAL_TRUE;
	}
	else
	{
		SENSORDB("@@@SP:Ronlus capture preview\r\n");
		NT99141_MPEG4_encode_mode = KAL_FALSE;
	}
	
    image_window->GrabStartX= 1;
    image_window->GrabStartY= 1;
    image_window->ExposureWindowWidth = IMAGE_SENSOR_PV_WIDTH;//800-16; //1280-16; //1600-32 ; //800-16; 
    image_window->ExposureWindowHeight = IMAGE_SENSOR_PV_HEIGHT;//600-12;//720-12; //1200-24; //600-12;

    NT99141_DELAY_AFTER_PREVIEW = 1;
	memcpy(&NT99141SensorConfigData, sensor_config_data, sizeof(MSDK_SENSOR_CONFIG_STRUCT));//rotation
	return ERROR_NONE;
} /* NT99141Preview */


/*************************************************************************
 * FUNCTION
 *	NT99141Capture
 *
 * DESCRIPTION
 *	This function setup the CMOS sensor in capture MY_OUTPUT mode
 *
 * PARAMETERS
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99141Capture(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
		MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)

{
	SENSORDB("line %d\r\n",__LINE__);
	NT99141_MODE_CAPTURE=KAL_TRUE;

    // config: UXGA for capture
    NT99141_config_window();
	
     if ((image_window->ImageTargetWidth<=640-16)&&
        (image_window->ImageTargetHeight<=480-12)){/* Less than PV Mode */
			image_window->GrabStartX = 0;
			image_window->GrabStartY = 1;
			image_window->ExposureWindowWidth= 640-16;
			image_window->ExposureWindowHeight = 480-12;
	  }
     else{
		//kal_uint32 shutter, cap_dummy_pixels = 0; 
		if(!setshutter){
			//// NT99141_BeforeSnapshot();
			image_window->GrabStartX = 1;//iStartX;
			image_window->GrabStartY = 1;//iStartY;
			image_window->ExposureWindowWidth = IMAGE_SENSOR_FULL_WIDTH; //16 ;
			image_window->ExposureWindowHeight = IMAGE_SENSOR_FULL_HEIGHT; //12;
		}
		else{
			image_window->GrabStartX = 1;
			image_window->GrabStartY = 1;
			image_window->ExposureWindowWidth = IMAGE_SENSOR_FULL_WIDTH; //16 ;
			image_window->ExposureWindowHeight = IMAGE_SENSOR_FULL_HEIGHT; //12;
		}

		setshutter = KAL_TRUE;	
	}

	 NT99141_DELAY_AFTER_PREVIEW = 2;
	// copy sensor_config_data
	memcpy(&NT99141SensorConfigData, sensor_config_data, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
	//SENSORDB("Ronlus NT99141Capture fun end\r\n");
	return ERROR_NONE;
} /* NT99141_Capture() */




UINT32 NT99141GetResolution(MSDK_SENSOR_RESOLUTION_INFO_STRUCT *pSensorResolution)
{
	SENSORDB("line %d\r\n",__LINE__);
    pSensorResolution->SensorFullWidth = IMAGE_SENSOR_FULL_WIDTH; //1600-96; //-16-32;
    pSensorResolution->SensorFullHeight = IMAGE_SENSOR_FULL_HEIGHT; //1200-72; //12-24;
    pSensorResolution->SensorPreviewWidth = IMAGE_SENSOR_PV_WIDTH; //1600-64; //800-16-8;//IMAGE_SENSOR_SVGA_WIDTH;(user this for SVGA)
    pSensorResolution->SensorPreviewHeight = IMAGE_SENSOR_PV_HEIGHT; //1200-48; //600-12-6;//IMAGE_SIMAGE_SENSOR_SVGA_HEIGHT;(use this for SVGA)
	pSensorResolution->SensorVideoWidth = IMAGE_SENSOR_FULL_WIDTH; //1600-64;  //800-16-8;
	pSensorResolution->SensorVideoHeight = IMAGE_SENSOR_FULL_HEIGHT; //1200-48; //600-12-6;
	
    pSensorResolution->SensorHighSpeedVideoWidth = IMAGE_SENSOR_PV_WIDTH;
    pSensorResolution->SensorHighSpeedVideoHeight = IMAGE_SENSOR_PV_HEIGHT;
    
    pSensorResolution->SensorSlimVideoWidth = IMAGE_SENSOR_PV_WIDTH;
    pSensorResolution->SensorSlimVideoHeight = IMAGE_SENSOR_PV_HEIGHT;  
	return ERROR_NONE;
} /* NT99141GetResolution() */ 


UINT32 NT99141GetInfo(MSDK_SCENARIO_ID_ENUM ScenarioId,
		MSDK_SENSOR_INFO_STRUCT *pSensorInfo,
		MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData)
{
	LOG_DEBUG("line %d\r\n",__LINE__);
    pSensorInfo->SensorPreviewResolutionX = IMAGE_SENSOR_PV_WIDTH; //1600-64; //800-16-8;//IMAGE_SENSOR_PV_WIDTH(use this for SVGA);
    pSensorInfo->SensorPreviewResolutionY = IMAGE_SENSOR_PV_HEIGHT; //1200-48; //600-12-6;//IMAGE_SENSOR_PV_HEIGHT(user this for SVGA);
    pSensorInfo->SensorFullResolutionX = IMAGE_SENSOR_FULL_WIDTH; //1600-96; //-16-32;
    pSensorInfo->SensorFullResolutionY = IMAGE_SENSOR_FULL_HEIGHT; //1200-72; //-12-24;

	pSensorInfo->SensorCameraPreviewFrameRate=g_imgsensor_fps?g_imgsensor_fps/10:30;  
	pSensorInfo->SensorVideoFrameRate=g_imgsensor_fps?g_imgsensor_fps/10:30;
	pSensorInfo->SensorStillCaptureFrameRate=g_imgsensor_fps?g_imgsensor_fps/10/2:10;
	pSensorInfo->SensorWebCamCaptureFrameRate=g_imgsensor_fps?g_imgsensor_fps/10/2:15; 
	pSensorInfo->SensorResetActiveHigh=TRUE;
	pSensorInfo->SensorResetDelayCount=1;
	pSensorInfo->SensorOutputDataFormat=SENSOR_OUTPUT_FORMAT_UYVY;
	pSensorInfo->SensorClockPolarity=SENSOR_CLOCK_POLARITY_LOW;
	pSensorInfo->SensorClockFallingPolarity=SENSOR_CLOCK_POLARITY_LOW;
	
	pSensorInfo->SensorHsyncPolarity =SENSOR_CLOCK_POLARITY_LOW;
	pSensorInfo->SensorVsyncPolarity = SENSOR_CLOCK_POLARITY_HIGH;
	pSensorInfo->SensorInterruptDelayLines = 1;
	pSensorInfo->SensroInterfaceType=SENSOR_INTERFACE_TYPE_PARALLEL;
  
	pSensorInfo->CaptureDelayFrame = 1;
	pSensorInfo->PreviewDelayFrame = 1;
	pSensorInfo->VideoDelayFrame = 4;
	pSensorInfo->SensorMasterClockSwitch = 0;
	pSensorInfo->SensorDrivingCurrent = ISP_DRIVING_8MA;///ISP_DRIVING_2MA; //modify by sp_yjp,20130813
	
	pSensorInfo->HighSpeedVideoDelayFrame = 4;
	pSensorInfo->SlimVideoDelayFrame = 4;
 	pSensorInfo->SensorModeNum = 5;


	switch (ScenarioId)
	{
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			//case MSDK_SCENARIO_ID_VIDEO_CAPTURE_MPEG4:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount=	3;
			pSensorInfo->SensorClockRisingCount= 0;
			pSensorInfo->SensorClockFallingCount= 2;
			pSensorInfo->SensorPixelClockCount= 3;
			pSensorInfo->SensorDataLatchCount= 2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;

			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			//  case MSDK_SCENARIO_ID_CAMERA_CAPTURE_MEM:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount= 3;
			pSensorInfo->SensorClockRisingCount=0;
			pSensorInfo->SensorClockFallingCount=2;
			pSensorInfo->SensorPixelClockCount=3;
			pSensorInfo->SensorDataLatchCount=2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;
			break;
		default:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount= 3;
			pSensorInfo->SensorClockRisingCount=0;
			pSensorInfo->SensorClockFallingCount=2;
			pSensorInfo->SensorPixelClockCount=3;
			pSensorInfo->SensorDataLatchCount=2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;
			break;
	}
	NT99141PixelClockDivider=pSensorInfo->SensorPixelClockCount;
	memcpy(pSensorConfigData, &NT99141SensorConfigData, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
	return ERROR_NONE;
} /* NT99141GetInfo() */


UINT32 NT99141Control(MSDK_SCENARIO_ID_ENUM ScenarioId, MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *pImageWindow,
		MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData)
{
	//SENSORDB("line %d\r\n",__LINE__);
	switch (ScenarioId)
	{
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			//case MSDK_SCENARIO_ID_VIDEO_CAPTURE_MPEG4:
			NT99141Preview(pImageWindow, pSensorConfigData);
			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			//case MSDK_SCENARIO_ID_CAMERA_CAPTURE_MEM:
			NT99141Capture(pImageWindow, pSensorConfigData);
			break;
		default:
            SENSORDB("Error ScenarioId setting");
            NT99141Preview(pImageWindow, pSensorConfigData);
            return ERROR_INVALID_SCENARIO_ID;
	}


	return TRUE;
}	/* NT99141Control() */

BOOL NT99141_set_param_wb(UINT16 para)
{

	if(fromsd)  return true;
	
#ifdef NT99141YUV_DEBUG
	 SENSORDB("line %d para = %d\r\n",__LINE__,para);

	switch (para)
	{
		case AWB_MODE_OFF:
			NT99141_write_cmos_sensor(0x3201,0x6F);//bit4
			NT99141_write_cmos_sensor(0x3060,0x01); 
			break; 					
		case AWB_MODE_AUTO:
			NT99141_write_cmos_sensor(0x3201,0x7F);
			NT99141_write_cmos_sensor(0x3060,0x01); 
			break;
		case AWB_MODE_CLOUDY_DAYLIGHT: //cloudy
			NT99141_write_cmos_sensor(0x3290,0x01);	// 337 / 256 = 1.316								 
			NT99141_write_cmos_sensor(0x3291,0x51);													   
			NT99141_write_cmos_sensor(0x3296,0x01);	// 256 / 256 = 1.0									 
			NT99141_write_cmos_sensor(0x3297,0x00);													   
			NT99141_write_cmos_sensor(0x3060,0x01);

			break;
		case AWB_MODE_DAYLIGHT: //sunny
			NT99141_write_cmos_sensor(0x3290,0x01);	// 312 / 256 = 1.218								 
			NT99141_write_cmos_sensor(0x3291,0x67);													   
			NT99141_write_cmos_sensor(0x3296,0x01);	// 360 / 256 = 1.406								 
			NT99141_write_cmos_sensor(0x3297,0x3d);													   
			NT99141_write_cmos_sensor(0x3060,0x01); 

			break;
		case AWB_MODE_INCANDESCENT: //office
			NT99141_write_cmos_sensor(0x3290,0x00);	// 304 / 256 = 1.187								 
			NT99141_write_cmos_sensor(0x3291,0xe4);													   
			NT99141_write_cmos_sensor(0x3296,0x01);	// 459 / 256 = 1.793								 
			NT99141_write_cmos_sensor(0x3297,0xe0);													   
			NT99141_write_cmos_sensor(0x3060,0x01);

			break;
		case AWB_MODE_TUNGSTEN: //home
			NT99141_write_cmos_sensor(0x3290,0x00);	// 256 / 256 = 1.0									 
			NT99141_write_cmos_sensor(0x3291,0xe6);													   
			NT99141_write_cmos_sensor(0x3296,0x02);	// 560 / 256 = 2.1875								 
			NT99141_write_cmos_sensor(0x3297,0xe0);													   
			NT99141_write_cmos_sensor(0x3060,0x01);

			break;
		case AWB_MODE_FLUORESCENT:
			NT99141_write_cmos_sensor(0x3290,0x01);	// 368 / 256 = 1.437								 
			NT99141_write_cmos_sensor(0x3291,0x1d);													   
			NT99141_write_cmos_sensor(0x3296,0x01);	// 511 / 256 = 1.996								 
			NT99141_write_cmos_sensor(0x3297,0xb4);													   
			NT99141_write_cmos_sensor(0x3060,0x01); 

		default:
			return FALSE;
	}
#endif
	return TRUE;
} /* NT99141_set_param_wb */


BOOL NT99141_set_param_effect(UINT16 para)
{
	if(fromsd)  return true;
	
	SENSORDB("line %d para %d\r\n",__LINE__,para);

#ifdef NT99141YUV_DEBUG	
	switch (para)
	{
		case MEFFECT_OFF:

			break;
		case MEFFECT_SEPIA:

			break;
		case MEFFECT_NEGATIVE://----datasheet

			break;
		case MEFFECT_SEPIAGREEN://----datasheet aqua

			break;
		case MEFFECT_SEPIABLUE:

			break;
		case MEFFECT_MONO: //----datasheet black & white

			break;
		default:
			return FALSE;
	}

	#endif 
	return TRUE;
} /* NT99141_set_param_effect */

//UINT8 index = 1;
BOOL NT99141_set_param_banding(UINT16 para)
{
	if(fromsd)  return TRUE;
	
#ifdef NT99141YUV_DEBUG
	SENSORDB("line %d para = %d\r\n",__LINE__,para);

	switch (para)
	{
		case AE_FLICKER_MODE_50HZ:
			NT99141_write_cmos_sensor(0x32BF,0x60); 
			NT99141_write_cmos_sensor(0x32C0,0x5B); 
			NT99141_write_cmos_sensor(0x32C1,0x5B); 
			NT99141_write_cmos_sensor(0x32C2,0x5B); 
			NT99141_write_cmos_sensor(0x32C3,0x00); 
			NT99141_write_cmos_sensor(0x32C4,0x27); 
			NT99141_write_cmos_sensor(0x32C5,0x28); 
			NT99141_write_cmos_sensor(0x32C6,0x28); 
			NT99141_write_cmos_sensor(0x32C7,0x00); 
			NT99141_write_cmos_sensor(0x32C8,0xD8); 
			NT99141_write_cmos_sensor(0x32C9,0x5B); 
			NT99141_write_cmos_sensor(0x32CA,0x83); 
			NT99141_write_cmos_sensor(0x32CB,0x83); 
			NT99141_write_cmos_sensor(0x32CC,0x83); 
			NT99141_write_cmos_sensor(0x32CD,0x83); 
			NT99141_write_cmos_sensor(0x32DB,0x7B); 
			NT99141_write_cmos_sensor(0x3060,0x01); 
			NT99141_CAM_BANDING_50HZ = KAL_TRUE;
			break;
		case AE_FLICKER_MODE_60HZ:
			NT99141_write_cmos_sensor(0x32BF,0x60); 
			NT99141_write_cmos_sensor(0x32C0,0x60); 
			NT99141_write_cmos_sensor(0x32C1,0x60); 
			NT99141_write_cmos_sensor(0x32C2,0x60); 
			NT99141_write_cmos_sensor(0x32C3,0x00); 
			NT99141_write_cmos_sensor(0x32C4,0x27); 
			NT99141_write_cmos_sensor(0x32C5,0x28); 
			NT99141_write_cmos_sensor(0x32C6,0x28); 
			NT99141_write_cmos_sensor(0x32C7,0x00); 
			NT99141_write_cmos_sensor(0x32C8,0xB4); 
			NT99141_write_cmos_sensor(0x32C9,0x60); 
			NT99141_write_cmos_sensor(0x32CA,0x88); 
			NT99141_write_cmos_sensor(0x32CB,0x88); 
			NT99141_write_cmos_sensor(0x32CC,0x88); 
			NT99141_write_cmos_sensor(0x32CD,0x88); 
			NT99141_write_cmos_sensor(0x32DB,0x76);
			NT99141_write_cmos_sensor(0x3060,0x01); 
			NT99141_CAM_BANDING_50HZ = KAL_FALSE;
			break;
		default:
			return FALSE;
	}

#endif 
	return TRUE;
} /* NT99141_set_param_banding */


BOOL NT99141_set_param_exposure(UINT16 para)
{
	if(fromsd)  return true;
	
	SENSORDB("line %d para %d\r\n",__LINE__,para);
#ifdef NT99141YUV_DEBUG
	switch (para)
	{
		case AE_EV_COMP_n30:              /* EV -2 */	 
			NT99141_write_cmos_sensor(0x32F2,0x3A); 
			break;
		case AE_EV_COMP_n20:              /* EV -1.5 */
			NT99141_write_cmos_sensor(0x32F2,0x4A);
			break;
		case AE_EV_COMP_n10:              /* EV -1 */
			NT99141_write_cmos_sensor(0x32F2,0x5A);

			break;
		case AE_EV_COMP_n03:   
			NT99141_write_cmos_sensor(0x32F2,0x6A);
			break;  
	
		case AE_EV_COMP_00:                /* EV 0 */
			NT99141_write_cmos_sensor(0x32F2,0x80);

			break;
		case AE_EV_COMP_10:              /* EV +0.5 */
			NT99141_write_cmos_sensor(0x32F2,0x8A);

			break;
		case AE_EV_COMP_13 :              /* EV +1 */
			NT99141_write_cmos_sensor(0x32F2,0x9A);

			break;
		case AE_EV_COMP_20 :              /* EV +1.5 */
			NT99141_write_cmos_sensor(0x32F2,0xAA);

			break;
		case AE_EV_COMP_30:   
			NT99141_write_cmos_sensor(0x32F2,0xBA);
			break;  
		default:
			return FALSE;
	}
	NT99141_write_cmos_sensor(0x3060,0x01); 

#endif 
	return TRUE;

} /* NT99141_set_param_exposure */


UINT32 NT99141YUVSensorSetting(FEATURE_ID iCmd, UINT16 iPara)
{
	LOG_DEBUG("line %d icmd=%d ipara=%d\r\n",__LINE__,iCmd,iPara);
	switch (iCmd) 
	{
		case FID_SCENE_MODE:	
			#if 0 //del by sp_yjp for test,20130822
			if (iPara == SCENE_MODE_OFF)
			{
				NT99141_night_mode(0); 
			}
			else if (iPara == SCENE_MODE_NIGHTSCENE)
			{
				NT99141_night_mode(1); 
			}
			#endif
			break; 	    
		case FID_AWB_MODE:
			NT99141_set_param_wb(iPara);
			break;
		case FID_COLOR_EFFECT:
			NT99141_set_param_effect(iPara);
			break;
		case FID_AE_EV:
			NT99141_set_param_exposure(iPara);
			break;
		case FID_AE_FLICKER:
			NT99141_set_param_banding(iPara);
			NT99141_night_mode(NT99141_CAM_Nightmode); 
			break;
		default:
			break;
	}
	return TRUE;
} /* NT99141YUVSensorSetting */

UINT32 nt99141_get_sensor_id(UINT32 *sensorID) 
{
	volatile signed char i;
	kal_uint16 sensor_id=0;
	SENSORDB("line %d\r\n",__LINE__);
	for(i=0;i<10;i++)
	{
		sensor_id = NT99141_read_cmos_sensor(0x3000);
		sensor_id = (sensor_id << 8) | NT99141_read_cmos_sensor(0x3001);
		if(sensor_id == NT99141_YUV_SENSOR_ID)
			break;
	}

	if(sensor_id != NT99141_YUV_SENSOR_ID)
	{
		pr_err("@@@SP:>>>>>>>>>>NT99141 read ID error<<<<<<<<<<<\n");			
		*sensorID = 0xFFFFFFFF;
		return ERROR_SENSOR_CONNECT_FAIL;
	}
	
	*sensorID = sensor_id;
	SENSORDB("@@@SP:NT99141GetSensorID=%#x\r\n ",sensor_id);
	return ERROR_NONE;
	
}


UINT32 NT99141_SetTestPatternMode(kal_bool bEnable)
{
    SENSORDB("bEnable: %d", bEnable);

    if(bEnable) 
	{
	}
	else        
	{
	}
    return ERROR_NONE;
}

#define NT99141_TEST_PATTERN_CHECKSUM (0x3f4b965a)
UINT32 NT99141FeatureControl(MSDK_SENSOR_FEATURE_ENUM FeatureId,
		UINT8 *pFeaturePara,UINT32 *pFeatureParaLen)
{
	UINT16 *pFeatureReturnPara16=(UINT16 *) pFeaturePara;
//	UINT16 *pFeatureData16=(UINT16 *) pFeaturePara;
	UINT32 *pFeatureReturnPara32=(UINT32 *) pFeaturePara;
	UINT32 *pFeatureData32=(UINT32 *) pFeaturePara;
	
    unsigned long long *pFeatureData=(unsigned long long *) pFeaturePara;
//  unsigned long long *pFeatureReturnPara=(unsigned long long *) pFeaturePara;
//	UINT32 NT99141SensorRegNumber;
//	UINT32 i;

	SENSOR_WINSIZE_INFO_STRUCT *wininfo;
	MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData=(MSDK_SENSOR_CONFIG_STRUCT *) pFeaturePara;
	MSDK_SENSOR_REG_INFO_STRUCT *pSensorRegData=(MSDK_SENSOR_REG_INFO_STRUCT *) pFeaturePara;

	LOG_DEBUG("line %d  FeatureId=%d\r\n",__LINE__,FeatureId);
	switch (FeatureId)
	{
		case SENSOR_FEATURE_GET_RESOLUTION:
			*pFeatureReturnPara16++=N720P_PERIOD_PIXEL_NUMS;
			*pFeatureReturnPara16=N720P_PERIOD_LINE_NUMS;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_GET_PERIOD:
			*pFeatureReturnPara16++=(SVGA_PERIOD_PIXEL_NUMS)+NT99141_dummy_pixels;
			*pFeatureReturnPara16=(SVGA_PERIOD_LINE_NUMS)+NT99141_dummy_lines;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_GET_PIXEL_CLOCK_FREQ:
			*pFeatureReturnPara32 = NT99141_g_fPV_PCLK;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_SET_ESHUTTER:
			break;
		case SENSOR_FEATURE_SET_NIGHTMODE:
#ifndef DEBUG_SENSOR_NT99141		
			NT99141_night_mode((BOOL) *pFeatureData);
#endif
			break;
		case SENSOR_FEATURE_SET_GAIN:
		case SENSOR_FEATURE_SET_FLASHLIGHT:
			break;
		case SENSOR_FEATURE_SET_ISP_MASTER_CLOCK_FREQ:
			NT99141_isp_master_clock=*pFeatureData32;
			break;
		case SENSOR_FEATURE_SET_REGISTER:
			if(pSensorRegData->RegAddr == 0xEF01 ){
				g_imgsensor_fps  =  pSensorRegData->RegData*50;
				//SENSORDB("set fps to %d\n", pSensorRegData->RegData*50);
				NT99141_config_window();
			}else
				NT99141_write_cmos_sensor(pSensorRegData->RegAddr, pSensorRegData->RegData);
			
			break;
		case SENSOR_FEATURE_GET_REGISTER:
			if(pSensorRegData->RegAddr == 0xEF01 )
				pSensorRegData->RegData = g_imgsensor_fps;
			else 
				pSensorRegData->RegData = NT99141_read_cmos_sensor(pSensorRegData->RegAddr);
			break;
			
		case SENSOR_FEATURE_GET_CONFIG_PARA:
			memcpy(pSensorConfigData, &NT99141SensorConfigData, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
			*pFeatureParaLen=sizeof(MSDK_SENSOR_CONFIG_STRUCT);
			break;
			
		case SENSOR_FEATURE_SET_FRAMERATE:
			//SENSORDB("current fps :%d\n", (int)*pFeatureData);
			g_imgsensor_fps = *pFeatureData;
			NT99141_config_window();//Immediate execution
			
		break;
		case SENSOR_FEATURE_SET_CCT_REGISTER:
		case SENSOR_FEATURE_GET_CCT_REGISTER:
		case SENSOR_FEATURE_SET_ENG_REGISTER:
		case SENSOR_FEATURE_GET_ENG_REGISTER:
		case SENSOR_FEATURE_GET_REGISTER_DEFAULT:
		case SENSOR_FEATURE_CAMERA_PARA_TO_SENSOR:
		case SENSOR_FEATURE_SENSOR_TO_CAMERA_PARA:
		case SENSOR_FEATURE_GET_GROUP_COUNT:
		case SENSOR_FEATURE_GET_GROUP_INFO:
		case SENSOR_FEATURE_GET_ITEM_INFO:
		case SENSOR_FEATURE_SET_ITEM_INFO:
		case SENSOR_FEATURE_GET_ENG_INFO:
			break;
		case SENSOR_FEATURE_CHECK_SENSOR_ID:
			nt99141_get_sensor_id(pFeatureReturnPara32); 
			break; 
		case SENSOR_FEATURE_GET_LENS_DRIVER_ID:
			// get the lens driver ID from EEPROM or just return LENS_DRIVER_ID_DO_NOT_CARE
			// if EEPROM does not exist in camera module.
			*pFeatureReturnPara32=LENS_DRIVER_ID_DO_NOT_CARE;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_SET_YUV_CMD:
			NT99141YUVSensorSetting((FEATURE_ID)*pFeatureData, *(pFeatureData+1));//test ,del by sp_yjp,20130813
			break;
		case SENSOR_FEATURE_SET_TEST_PATTERN:
        	NT99141_SetTestPatternMode((BOOL)*pFeatureData);
       		break;
    	case SENSOR_FEATURE_GET_TEST_PATTERN_CHECKSUM_VALUE://for factory mode auto testing             
        	*pFeatureReturnPara32= NT99141_TEST_PATTERN_CHECKSUM;
        	*pFeatureParaLen=4; 
			break;
		case SENSOR_FEATURE_GET_CROP_INFO:
            //LOG_INF("SENSOR_FEATURE_GET_CROP_INFO scenarioId:%d\n", *pFeatureData);
            // For pointer address in user space is 32-bit, need use compat_ptr to convert it to 64-bit in kernel space    

            wininfo = (SENSOR_WINSIZE_INFO_STRUCT *)(uintptr_t)(*(pFeatureData+1));

            switch (*pFeatureData32) {
                case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[1],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[2],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[3],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_SLIM_VIDEO:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[4],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
                default:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[0],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
            }
			break;
			
		default:
			break;
	}
	return ERROR_NONE;
}	/* NT99141FeatureControl() */


SENSOR_FUNCTION_STRUCT	SensorFuncNT99141YUV=
{
	NT99141Open,
	NT99141GetInfo,
	NT99141GetResolution,
	NT99141FeatureControl,
	NT99141Control,
	NT99141Close
};


UINT32 NT99141_YUV_SensorInit(PSENSOR_FUNCTION_STRUCT *pfFunc)
{
	/* To Do : Check Sensor status here */	
	if (pfFunc!=NULL)
	{
		SENSORDB("@@@SP:NT99141_YUV_SensorInit\r\n");
		*pfFunc=&SensorFuncNT99141YUV;
	}
	return ERROR_NONE;
} /* SensorInit() */
