/*
 * Copyright (C) 2015 MediaTek Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */

/*****************************************************************************
 *
 * Filename:
 * ---------
 *   nt99230mipiyuv_Sensor.c
 *
 * Project:
 * --------
 *   MAUI
 *
 * Description:
 * ------------
 *   Image sensor driver function
 *   V1.0.0
 *
 * Author:
 * -------
 *   Mormo
 *
 *=============================================================
 *             HISTORY
 * Below this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *------------------------------------------------------------------------------
 * $Log$
 * 2011/11/03 Firsty Released By Mormo;
 *   
 *
 *------------------------------------------------------------------------------
 * Upper this line, this part is controlled by PVCS VM. DO NOT MODIFY!!
 *=============================================================
 ******************************************************************************/
#include <linux/videodev2.h>
#include <linux/i2c.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/cdev.h>
#include <linux/uaccess.h>
#include <linux/fs.h>
#include <asm/atomic.h>
#include "kd_camera_hw.h"
#include "kd_imgsensor.h"
#include "kd_imgsensor_define.h"
#include "kd_imgsensor_errcode.h"
#include "kd_camera_feature.h"
#include "nt99230mipiyuv_Sensor.h"
#include "nt99230mipiyuv_Camera_Sensor_para.h"
#include "nt99230mipiyuv_CameraCustomized.h"

/*******************************************************************************
 * // Adapter for Winmo typedef
 ********************************************************************************/
#define WINMO_USE 0

#define Sleep(ms) mdelay(ms)
#define RETAILMSG(x,...)
#define TEXT

#define PFX         "NT99230YUV"
#define DEBUG	1
#if DEBUG
    #define LOG_INF(format, args...)	pr_err( PFX "[%s] " format, __FUNCTION__, ##args)
#else
    #define LOG_INF(format, args...)   
#endif

//#define DEBUG_SENSOR_NT99230

//#define NT99230_24M_72M

#define PREVIEW_USE_VGA     0
#define PREVIEW_USE_SVGA    0 // 1
#define PREVIEW_USE_720P    0 // 0
#define PREVIEW_USE_1080P   1 // 0

#if PREVIEW_USE_VGA
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_VGA_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_VGA_HEIGHT - 6)
#endif

#if PREVIEW_USE_SVGA
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_SVGA_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_SVGA_HEIGHT - 6)
#endif

#if PREVIEW_USE_720P
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_720P_WIDTH - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_720P_HEIGHT - 6)
#endif

#if PREVIEW_USE_1080P
#undef IMAGE_SENSOR_PV_WIDTH
#undef IMAGE_SENSOR_PV_HEIGHT
//#define IMAGE_SENSOR_PV_WIDTH					(IMAGE_SENSOR_1080P_WIDTH - 8)
//#define IMAGE_SENSOR_PV_HEIGHT					(IMAGE_SENSOR_1080P_HEIGHT - 6)
#define IMAGE_SENSOR_PV_WIDTH					(640 - 8)
#define IMAGE_SENSOR_PV_HEIGHT					(480 - 6)

#endif

#ifdef DEBUG_SENSOR_NT99230

kal_uint8 fromsd = 0;
kal_uint16 NT99230_write_cmos_sensor(kal_uint8 addr, kal_uint8 para);
kal_uint16 NT99230_read_cmos_sensor(kal_uint8 addr);

#define NT99230_OP_CODE_INI		0x00		/* Initial value. */
#define NT99230_OP_CODE_REG		0x01		/* Register */
#define NT99230_OP_CODE_DLY		0x02		/* Delay */
#define NT99230_OP_CODE_END		0x03		/* End of initial setting. */
	

typedef struct
{
	u16 init_reg;
	u16 init_val;	/* Save the register value and delay tick */
	u8 op_code;		/* 0 - Initial value, 1 - Register, 2 - Delay, 3 - End of setting. */
} NT99230_initial_set_struct;

NT99230_initial_set_struct NT99230_Init_Reg[1000];

u32 strtol(const char *nptr, u8 base)
{
	u8 ret;
	if(!nptr || (base!=16 && base!=10 && base!=8))
	{
		printk("%s(): NULL pointer input\n", __FUNCTION__);
		return -1;
	}
	for(ret=0; *nptr; nptr++)
	{
		if((base==16 && *nptr>='A' && *nptr<='F') || 
				(base==16 && *nptr>='a' && *nptr<='f') || 
				(base>=10 && *nptr>='0' && *nptr<='9') ||
				(base>=8 && *nptr>='0' && *nptr<='7') )
		{
			ret *= base;
			if(base==16 && *nptr>='A' && *nptr<='F')
				ret += *nptr-'A'+10;
			else if(base==16 && *nptr>='a' && *nptr<='f')
				ret += *nptr-'a'+10;
			else if(base>=10 && *nptr>='0' && *nptr<='9')
				ret += *nptr-'0';
			else if(base>=8 && *nptr>='0' && *nptr<='7')
				ret += *nptr-'0';
		}
		else
			return ret;
	}
	return ret;
}

u8 NT99230_Initialize_from_T_Flash()
{
	//FS_HANDLE fp = -1;				/* Default, no file opened. */
	//u8 *data_buff = NULL;
	u8 *curr_ptr = NULL;
	u32 file_size = 0;
	//u32 bytes_read = 0;
	u32 i = 0, j = 0;
	u8 func_ind[4] = {0};	/* REG or DLY */


	struct file *fp; 
	mm_segment_t fs; 
	loff_t pos = 0; 
	static u8 data_buff[10*1024] ;

    fp = filp_open("/storage/sdcard1/nt99230_sd.dat", O_RDONLY , 0); 
    if (IS_ERR(fp)) { 
		printk("create file error 0\n");
		fp = filp_open("/storage/sdcard0/nt99230_sd.dat", O_RDONLY , 0); 
    	if (IS_ERR(fp)) { 
	        printk("create file error 1\n"); 
	        return 2;//-1; 
    	}
    } 
    else
		printk("NT99230_Initialize_from_T_Flash Open File Success\n");
	fs = get_fs(); 
	set_fs(KERNEL_DS); 

	file_size = vfs_llseek(fp, 0, SEEK_END);
	vfs_read(fp, data_buff, file_size, &pos); 
	//printk("%s %d %d\n", buf,iFileLen,pos); 
	filp_close(fp, NULL); 
	set_fs(fs);


	/* Start parse the setting witch read from t-flash. */
	curr_ptr = data_buff;
	while (curr_ptr < (data_buff + file_size))
	{
		while ((*curr_ptr == ' ') || (*curr_ptr == '\t'))/* Skip the Space & TAB */
			curr_ptr++;				

		if (((*curr_ptr) == '/') && ((*(curr_ptr + 1)) == '*'))
		{
			while (!(((*curr_ptr) == '*') && ((*(curr_ptr + 1)) == '/')))
			{
				curr_ptr++;		/* Skip block comment code. */
			}

			while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
			{
				curr_ptr++;
			}

			curr_ptr += 2;						/* Skip the enter line */

			continue ;
		}

		if (((*curr_ptr) == '/') || ((*curr_ptr) == '{') || ((*curr_ptr) == '}'))		/* Comment line, skip it. */
		{
			while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
			{
				curr_ptr++;
			}

			curr_ptr += 2;						/* Skip the enter line */

			continue ;
		}
		/* This just content one enter line. */
		if (((*curr_ptr) == 0x0D) && ((*(curr_ptr + 1)) == 0x0A))
		{
			curr_ptr += 2;
			continue ;
		}
		//printk(" curr_ptr1 = %s\n",curr_ptr);
		memcpy(func_ind, curr_ptr, 3);


		if (strcmp((const char *)func_ind, "REG") == 0)		/* REG */
		{
			curr_ptr += 6;				/* Skip "REG(0x" or "DLY(" */
			NT99230_Init_Reg[i].op_code = NT99230_OP_CODE_REG;

			NT99230_Init_Reg[i].init_reg = strtol((const char *)curr_ptr, 16);
			curr_ptr += 5;	/* Skip "00, 0x" */

			NT99230_Init_Reg[i].init_val = strtol((const char *)curr_ptr, 16);
			curr_ptr += 4;	/* Skip "00);" */

		}
		else									/* DLY */
		{
			/* Need add delay for this setting. */
			curr_ptr += 4;	
			NT99230_Init_Reg[i].op_code = NT99230_OP_CODE_DLY;

			NT99230_Init_Reg[i].init_reg = 0xFF;
			NT99230_Init_Reg[i].init_val = strtol((const char *)curr_ptr,  10);	/* Get the delay ticks, the delay should less then 50 */
		}
		i++;


		/* Skip to next line directly. */
		while (!((*curr_ptr == 0x0D) && (*(curr_ptr+1) == 0x0A)))
		{
			curr_ptr++;
		}
		curr_ptr += 2;
	}

	/* (0xFFFF, 0xFFFF) means the end of initial setting. */
	NT99230_Init_Reg[i].op_code = NT99230_OP_CODE_END;
	NT99230_Init_Reg[i].init_reg = 0xFF;
	NT99230_Init_Reg[i].init_val = 0xFF;
	i++;
	//for (j=0; j<i; j++)
	//printk(" %x  ==  %x\n",NT99230_Init_Reg[j].init_reg, NT99230_Init_Reg[j].init_val);

	/* Start apply the initial setting to sensor. */
#if 1
	for (j=0; j<i; j++)
	{
		if (NT99230_Init_Reg[j].op_code == NT99230_OP_CODE_END)	/* End of the setting. */
		{
			break ;
		}
		else if (NT99230_Init_Reg[j].op_code == NT99230_OP_CODE_DLY)
		{
			msleep(NT99230_Init_Reg[j].init_val);		/* Delay */
		}
		else if (NT99230_Init_Reg[j].op_code == NT99230_OP_CODE_REG)
		{

			NT99230_write_cmos_sensor((kal_uint8)NT99230_Init_Reg[j].init_reg, (kal_uint8)NT99230_Init_Reg[j].init_val);
		}
		else
		{
			printk("REG ERROR!\n");
		}
	}
#endif
	return 1;	
}

#endif
//auto lum
#define NT99230_NORMAL_Y0ffset  0x18 //0x10
#define NT99230_LOWLIGHT_Y0ffset  0x20

kal_bool   NT99230_MPEG4_encode_mode = KAL_FALSE;
kal_uint16 NT99230_dummy_pixels = 0, NT99230_dummy_lines = 0;
kal_bool   NT99230_MODE_CAPTURE = KAL_FALSE;
static kal_bool NT99230_gPVmode = KAL_TRUE; //PV size or Full size


static kal_int8 NT99230_DELAY_AFTER_PREVIEW = -1;

kal_bool   NT99230_CAM_BANDING_50HZ = KAL_TRUE;//lj_test KAL_FALSE;
kal_bool   NT99230_CAM_Nightmode = 0;
kal_bool	setshutter = KAL_FALSE;
kal_uint32 NT99230_isp_master_clock;
static kal_uint32 NT99230_g_fPV_PCLK = 24;

kal_uint8 NT99230_sensor_write_I2C_address = NT99230_WRITE_ID;
kal_uint8 NT99230_sensor_read_I2C_address = NT99230_READ_ID;

UINT8 NT99230PixelClockDivider=0;

MSDK_SENSOR_CONFIG_STRUCT NT99230SensorConfigData;

#define NT99230_SET_PAGE0 	NT99230_write_cmos_sensor(0xfd, 0x00)
#define NT99230_SET_PAGE1 	NT99230_write_cmos_sensor(0xfd, 0x01)

#define NT99230YUV_DEBUG
#ifdef NT99230YUV_DEBUG
#define SENSORDB(fmt, args...)  pr_err(PFX "%s: " fmt, __FUNCTION__ ,##args)
#else
#define SENSORDB(x,...)
#endif

#define WINDOW_SIZE_720P 	0
#define WINDOW_SIZE_SVGA 	0
#define WINDOW_SIZE_VGA	0
#define WINDOW_SIZE_1080P 1

extern int iReadRegI2C(u8 *a_pSendData , u16 a_sizeSendData, u8 * a_pRecvData, u16 a_sizeRecvData, u16 i2cId);
extern int iWriteRegI2C(u8 *a_pSendData , u16 a_sizeSendData, u16 i2cId);


/* Sensor output window information */


static SENSOR_WINSIZE_INFO_STRUCT imgsensor_winsize_info[5]=
{{ 1920, 1080,      0,    0, 1920, 1080, 1920,  1080,      0,   0, 640,  1080,  8,    6, 1904,  1068},  // Preview
 { 1920, 1080,      0,    0, 1920, 1080, 1920,  1080,      0,   0, 1920,  1080,  8,    6, 1904,  1068}, // capture
 { 1920, 1080,      0,    0, 1920, 1080, 1920,  1080,      0,   0, 1920,  1080,  8,    6, 1904,  1068},  // video
 { 1920, 1080,      0,    0, 1920, 1080, 1920,  1080,      0,   0, 1920,  1080,  160,    240, 1920,  1080}, //hight speed video
 { 1920, 1080,      0,    0, 1920, 1080, 1920,  1080,      0,   0, 1920,  1080,  160,    240, 1920,  1080}}; // slim video


void NT99230_write_cmos_sensor(kal_uint16 addr, kal_uint8 para,int i2caddr)
{
	char puSendCmd[4] = {(char)((addr >> 8) & 0xFF) ,(char)(addr & 0xFF), (char)(para & 0xFF)};

	kdSetI2CSpeed(400);
	iWriteRegI2C(puSendCmd , 3, i2caddr);

}
kal_uint8 NT99230_read_cmos_sensor(kal_uint16 addr)
{
	kal_uint8 get_byte=0;
	char puSendCmd[2] = { (char)((addr >> 8) & 0xFF) ,(char)((addr  >> 0)& 0xFF) };

	kdSetI2CSpeed(400);
	iReadRegI2C(puSendCmd , 2, (u8*)&get_byte, 1, 0x36);
	
	return get_byte;
}

/*************************************************************************
 * FUNCTION
 *	NT99230_SetShutter
 *
 * DESCRIPTION
 *	This function set e-shutter of NT99230 to change exposure time.
 *
 * PARAMETERS
 *   iShutter : exposured lines
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99230_Set_Shutter(kal_uint16 iShutter)
{
	SENSORDB("Ronlus NT99230_Set_Shutter\r\n");
} /* Set_NT99230_Shutter */


/*************************************************************************
 * FUNCTION
 *	NT99230_read_Shutter
 *
 * DESCRIPTION
 *	This function read e-shutter of NT99230 .
 *
 * PARAMETERS
 *  None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
kal_uint16 NT99230_Read_Shutter(void)
{
	//kal_uint8 temp_reg1, temp_reg2;
	kal_uint16 shutter = 0;

	//temp_reg1 = NT99230_read_cmos_sensor(0x04);
	//temp_reg2 = NT99230_read_cmos_sensor(0x03);

	//shutter = (temp_reg1 & 0xFF) | (temp_reg2 << 8);
	SENSORDB("Ronlus NT99230_Read_Shutter\r\n");
	return shutter;
} /* NT99230_read_shutter */

#if 0

/*************************************************************************
 * FUNCTION
 *	NT99230_ae_enable
 *
 * DESCRIPTION
 *	This function enable or disable the awb (Auto White Balance).
 *
 * PARAMETERS
 *	1. kal_bool : KAL_TRUE - enable awb, KAL_FALSE - disable awb.
 *
 * RETURNS
 *	kal_bool : It means set awb right or not.
 *
 *************************************************************************/

static void NT99230_ae_enable(kal_bool enable)
{	 
	kal_uint16 temp_AWB_reg = 0;
	SENSORDB("Ronlus NT99230_ae_enable\r\n");
	temp_AWB_reg = NT99230_read_cmos_sensor(0x32);//Ronlus 20120511
	if (enable == 1)
	{
		NT99230_write_cmos_sensor(0x32, (temp_AWB_reg |0x05));
	}
	else if(enable == 0)
	{
		NT99230_write_cmos_sensor(0x32, (temp_AWB_reg & (~0x05)));
	}
}

/*************************************************************************
 * FUNCTION
 *	NT99230_awb_enable
 *
 * DESCRIPTION
 *	This function enable or disable the awb (Auto White Balance).
 *
 * PARAMETERS
 *	1. kal_bool : KAL_TRUE - enable awb, KAL_FALSE - disable awb.
 *
 * RETURNS
 *	kal_bool : It means set awb right or not.
 *
 *************************************************************************/
static void NT99230_awb_enable(kal_bool enalbe)
{	 
	//kal_uint16 temp_AWB_reg = 0;
	SENSORDB("Ronlus NT99230_awb_enable\r\n");
}


/*************************************************************************
 * FUNCTION
 *	NT99230_set_hb_shutter
 *
 * DESCRIPTION
 *	This function set the dummy pixels(Horizontal Blanking) when capturing, it can be
 *	used to adjust the frame rate  for back-end process.
 *	
 *	IMPORTANT NOTICE: the base shutter need re-calculate for some sensor, or else flicker may occur.
 *
 * PARAMETERS
 *	1. kal_uint32 : Dummy Pixels (Horizontal Blanking)
 *	2. kal_uint32 : shutter (Vertical Blanking)
 *
 * RETURNS
 *	None
 *
 *************************************************************************/
static void NT99230_set_hb_shutter(kal_uint32 hb_add,  kal_uint32 shutter)
{
//	kal_uint32 hb_ori, hb_total;
//	kal_uint32 temp_reg, banding_step;
	SENSORDB("Ronlus NT99230_set_hb_shutter\r\n");
}    /* NT99230_set_dummy */
#endif

/*************************************************************************
 * FUNCTION
 *	NT99230_config_window
 *
 * DESCRIPTION
 *	This function config the hardware window of NT99230 for getting specified
 *  data of that window.
 *
 * PARAMETERS
 *	start_x : start column of the interested window
 *  start_y : start row of the interested window
 *  width  : column widht of the itnerested window
 *  height : row depth of the itnerested window
 *
 * RETURNS
 *	the data that read from NT99230
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
//void NT99230_config_window(kal_uint16 startx, kal_uint16 starty, kal_uint16 width, kal_uint16 height)//Ronlus
void NT99230_config_window(kal_uint8 index)
{
	SENSORDB("Ronlus NT99230_config_window,index = %d\r\n",index);
	switch(index)
	{
		case WINDOW_SIZE_1080P://1080p@30fps
			NT99230_write_cmos_sensor(0x32BB, 0x67);
			NT99230_write_cmos_sensor(0x32BF, 0x60);
			NT99230_write_cmos_sensor(0x32C0, 0x5A);
			NT99230_write_cmos_sensor(0x32C1, 0x5A);
			NT99230_write_cmos_sensor(0x32C2, 0x5A);
			NT99230_write_cmos_sensor(0x32C3, 0x00);
			NT99230_write_cmos_sensor(0x32C4, 0x38);
			NT99230_write_cmos_sensor(0x32C5, 0x38);
			NT99230_write_cmos_sensor(0x32C6, 0x38);
			NT99230_write_cmos_sensor(0x32D3, 0x01);
			NT99230_write_cmos_sensor(0x32D4, 0x50);
			NT99230_write_cmos_sensor(0x32D5, 0x85);
			NT99230_write_cmos_sensor(0x32D6, 0x01);
			NT99230_write_cmos_sensor(0x32D7, 0x17);
			NT99230_write_cmos_sensor(0x32D8, 0x81);
			NT99230_write_cmos_sensor(0x32F0, 0x01);
			NT99230_write_cmos_sensor(0x3200, 0x7E);
			NT99230_write_cmos_sensor(0x3201, 0x3F);
			NT99230_write_cmos_sensor(0x302A, 0x80);
			NT99230_write_cmos_sensor(0x302B, 0x00);
			NT99230_write_cmos_sensor(0x302C, 0x0F);
			NT99230_write_cmos_sensor(0x302D, 0x01);
			NT99230_write_cmos_sensor(0x302E, 0x00);
			NT99230_write_cmos_sensor(0x302F, 0x07);
			NT99230_write_cmos_sensor(0x3022, 0x24);
			NT99230_write_cmos_sensor(0x3023, 0x24);
			NT99230_write_cmos_sensor(0x3002, 0x00);
			NT99230_write_cmos_sensor(0x3003, 0x08);
			NT99230_write_cmos_sensor(0x3004, 0x00);
			NT99230_write_cmos_sensor(0x3005, 0x02);
			NT99230_write_cmos_sensor(0x3006, 0x07);
			NT99230_write_cmos_sensor(0x3007, 0x87);
			NT99230_write_cmos_sensor(0x3008, 0x04);
			NT99230_write_cmos_sensor(0x3009, 0x39);
			NT99230_write_cmos_sensor(0x300A, 0x09);
			NT99230_write_cmos_sensor(0x300B, 0x6C);
			NT99230_write_cmos_sensor(0x300C, 0x04);
			NT99230_write_cmos_sensor(0x300D, 0x5F);
			NT99230_write_cmos_sensor(0x300E, 0x07);
			NT99230_write_cmos_sensor(0x300F, 0x80);
			NT99230_write_cmos_sensor(0x3010, 0x04);
			NT99230_write_cmos_sensor(0x3011, 0x38);
			NT99230_write_cmos_sensor(0x3508, 0x01);
			NT99230_write_cmos_sensor(0x3509, 0x77);
			NT99230_write_cmos_sensor(0x3500, 0xE4);
			NT99230_write_cmos_sensor(0x320A, 0x00);
			NT99230_write_cmos_sensor(0x3021, 0x02);
			NT99230_write_cmos_sensor(0x3060, 0x01);	
			break;
		default:
			break;
	}
} /* NT99230_config_window */



/*************************************************************************
 * FUNCTION
 *	NT99230_SetGain
 *
 * DESCRIPTION
 *	This function is to set global gain to sensor.
 *
 * PARAMETERS
 *   iGain : sensor global gain(base: 0x40)
 *
 * RETURNS
 *	the actually gain set to sensor.
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
kal_uint16 NT99230_SetGain(kal_uint16 iGain)
{
	SENSORDB("Ronlus NT99230_SetGain\r\n");
	return iGain;
}


/*************************************************************************
 * FUNCTION
 *	NT99230_NightMode
 *
 * DESCRIPTION
 *	This function night mode of NT99230.
 *
 * PARAMETERS
 *	bEnable: KAL_TRUE -> enable night mode, otherwise, disable night mode
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99230_night_mode(kal_bool bEnable)
{
} 

/*************************************************************************
 * FUNCTION
 *	NT99230_Write_More_Registers
 *
 * DESCRIPTION
 *	This function is served for FAE to modify the necessary Init Regs. Do not modify the regs
 *     in init_NT99230() directly.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
void NT99230_Write_More_Registers(void)
{
	//Ronlus this function is used for FAE debug
}

#if 0

/*************************************************************************
 * FUNCTION
 *	NT99230_PV_setting
 *
 * DESCRIPTION
 *	This function apply the preview mode setting, normal the preview size is 1/4 of full size.
 *	Ex. 2M (1600 x 1200)
 *	Preview: 800 x 600 (Use sub-sample or binning to acheive it)
 *	Full Size: 1600 x 1200 (Output every effective pixels.)
 *
 * PARAMETERS
 *	1. image_sensor_exposure_window_struct : Set the grab start x,y and width,height.
 *	2. image_sensor_config_struct : Current operation mode.
 *
 * RETURNS
 *	None
 *
 *************************************************************************/
static void NT99230_PV_setting(void)
{
	SENSORDB("Ronlus NT99230_PV_setting\r\n");

} /* NT99230_PV_setting */


/*************************************************************************
 * FUNCTION
 *	NT99230_CAP_setting
 *
 * DESCRIPTION
 *	This function apply the full size mode setting.
 *	Ex. 2M (1600 x 1200)
 *	Preview: 800 x 600 (Use sub-sample or binning to acheive it)
 *	Full Size: 1600 x 1200 (Output every effective pixels.)
 *
 * PARAMETERS
 *	1. image_sensor_exposure_window_struct : Set the grab start x,y and width,height.
 *	2. image_sensor_config_struct : Current operation mode.
 *
 * RETURNS
 *	None
 *
 *************************************************************************/
static void NT99230_CAP_setting(void)
{
	//kal_uint16 corr_r_offset,corr_g_offset,corr_b_offset,temp = 0;
	SENSORDB("Ronlus NT99230_CAP_setting\r\n");	
} /* NT99230_CAP_setting */

#endif




/*************************************************************************
 * FUNCTION
 *	NT99230_Sensor_Init
 *
 * DESCRIPTION
 *	This function apply all of the initial setting to sensor.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 *************************************************************************/

void NT99230_Sensor_Init(void)
{
	NT99230_write_cmos_sensor(0x3069, 0x02);
	NT99230_write_cmos_sensor(0x306A, 0x03);
	NT99230_write_cmos_sensor(0x3100, 0x03); 
	NT99230_write_cmos_sensor(0x3101, 0x00);
	NT99230_write_cmos_sensor(0x3102, 0x09);
	NT99230_write_cmos_sensor(0x3103, 0x09);
	NT99230_write_cmos_sensor(0x3104, 0x00);
	NT99230_write_cmos_sensor(0x3105, 0x03);
	NT99230_write_cmos_sensor(0x3106, 0x07);
	NT99230_write_cmos_sensor(0x3107, 0x30);
	NT99230_write_cmos_sensor(0x3108, 0x50);
	NT99230_write_cmos_sensor(0x3109, 0x02);
	NT99230_write_cmos_sensor(0x310A, 0x75);
	NT99230_write_cmos_sensor(0x310B, 0xC0);
	NT99230_write_cmos_sensor(0x310C, 0x00);
	NT99230_write_cmos_sensor(0x310D, 0x43);
	NT99230_write_cmos_sensor(0x310E, 0x01);
	NT99230_write_cmos_sensor(0x3110, 0x88);
	NT99230_write_cmos_sensor(0x3111, 0xCE);
	NT99230_write_cmos_sensor(0x3112, 0x88);
	NT99230_write_cmos_sensor(0x3113, 0x66);
	NT99230_write_cmos_sensor(0x3114, 0x33);
	NT99230_write_cmos_sensor(0x3115, 0x88);
	NT99230_write_cmos_sensor(0x3116, 0x86);
	NT99230_write_cmos_sensor(0x3118, 0xAF);
	NT99230_write_cmos_sensor(0x3119, 0xAF);  
	NT99230_write_cmos_sensor(0x311A, 0xAF);
	NT99230_write_cmos_sensor(0x303F, 0x32);  
	NT99230_write_cmos_sensor(0x3055, 0x01);  
	NT99230_write_cmos_sensor(0x3051, 0x3B);
	NT99230_write_cmos_sensor(0x3052, 0x0F);  	
	NT99230_write_cmos_sensor(0x3053, 0x00);  
	NT99230_write_cmos_sensor(0x3056, 0x32);  
	NT99230_write_cmos_sensor(0x308A, 0x10);  
	NT99230_write_cmos_sensor(0x308B, 0x2F);  
	NT99230_write_cmos_sensor(0x308C, 0x28);  
	NT99230_write_cmos_sensor(0x308D, 0x32);  
	NT99230_write_cmos_sensor(0x308E, 0x37);  
	NT99230_write_cmos_sensor(0x308F, 0x0E);  
	NT99230_write_cmos_sensor(0x30A0, 0x03);       
	NT99230_write_cmos_sensor(0x30A4, 0x1F);
	NT99230_write_cmos_sensor(0x350B, 0xFF);  
	NT99230_write_cmos_sensor(0x3514, 0x05);
	NT99230_write_cmos_sensor(0x3515, 0x01);
	NT99230_write_cmos_sensor(0x3518, 0x06);
	NT99230_write_cmos_sensor(0x3519, 0x00);
	NT99230_write_cmos_sensor(0x351A, 0xFF);
	NT99230_write_cmos_sensor(0x3530, 0x50);
	NT99230_write_cmos_sensor(0x3534, 0x00);
	NT99230_write_cmos_sensor(0x3512, 0x05);
	NT99230_write_cmos_sensor(0x3511, 0x09);
	NT99230_write_cmos_sensor(0x3513, 0x00);
	NT99230_write_cmos_sensor(0x3532, 0x35);
	NT99230_write_cmos_sensor(0x3533, 0x20);
	NT99230_write_cmos_sensor(0x3F14, 0xFF);  
	NT99230_write_cmos_sensor(0x3F03, 0x00);  
	NT99230_write_cmos_sensor(0x3F04, 0x00);  
	NT99230_write_cmos_sensor(0x3F05, 0x00);  
	NT99230_write_cmos_sensor(0x3F06, 0x00);  
	NT99230_write_cmos_sensor(0x3F07, 0x00);  
	NT99230_write_cmos_sensor(0x3F09, 0x03);  
	NT99230_write_cmos_sensor(0x3F0A, 0x00);  
	NT99230_write_cmos_sensor(0x3F0B, 0x00);  
	NT99230_write_cmos_sensor(0x3F0C, 0x00);  
	NT99230_write_cmos_sensor(0x3F0D, 0x00);  
	NT99230_write_cmos_sensor(0x3F0E, 0x00);  
	NT99230_write_cmos_sensor(0x3F0F, 0x00);  
	NT99230_write_cmos_sensor(0x3F10, 0x00);  
	NT99230_write_cmos_sensor(0x3F11, 0x08);  
	NT99230_write_cmos_sensor(0x3F12, 0x20);  
	NT99230_write_cmos_sensor(0x3F13, 0x40);  
	NT99230_write_cmos_sensor(0x3F00, 0x20);  
	NT99230_write_cmos_sensor(0x3210, 0x1E);  
	NT99230_write_cmos_sensor(0x3211, 0x1C);
	NT99230_write_cmos_sensor(0x3212, 0x1E);
	NT99230_write_cmos_sensor(0x3213, 0x1E);
	NT99230_write_cmos_sensor(0x3214, 0x10);
	NT99230_write_cmos_sensor(0x3215, 0x10);
	NT99230_write_cmos_sensor(0x3216, 0x10);
	NT99230_write_cmos_sensor(0x3217, 0x10);
	NT99230_write_cmos_sensor(0x3218, 0x10);
	NT99230_write_cmos_sensor(0x3219, 0x10);
	NT99230_write_cmos_sensor(0x321A, 0x10);
	NT99230_write_cmos_sensor(0x321B, 0x10);
	NT99230_write_cmos_sensor(0x321C, 0x1A);
	NT99230_write_cmos_sensor(0x321D, 0x1C);
	NT99230_write_cmos_sensor(0x321E, 0x18);
	NT99230_write_cmos_sensor(0x321F, 0x18);
	NT99230_write_cmos_sensor(0x3230, 0x00);
	NT99230_write_cmos_sensor(0x3231, 0x00);
	NT99230_write_cmos_sensor(0x3232, 0x00);
	NT99230_write_cmos_sensor(0x3233, 0x0E);
	NT99230_write_cmos_sensor(0x3234, 0x00);
	NT99230_write_cmos_sensor(0x3235, 0x00);
	NT99230_write_cmos_sensor(0x3236, 0x00);
	NT99230_write_cmos_sensor(0x3237, 0x00);
	NT99230_write_cmos_sensor(0x3238, 0x18);
	NT99230_write_cmos_sensor(0x3239, 0x18);
	NT99230_write_cmos_sensor(0x323A, 0x30);
	NT99230_write_cmos_sensor(0x3243, 0xC4);
	NT99230_write_cmos_sensor(0x3244, 0x20);  
	NT99230_write_cmos_sensor(0x3245, 0x00);
	NT99230_write_cmos_sensor(0x3241, 0x45);	
	NT99230_write_cmos_sensor(0x3270, 0x08);  
	NT99230_write_cmos_sensor(0x3271, 0x14);
	NT99230_write_cmos_sensor(0x3272, 0x20);
	NT99230_write_cmos_sensor(0x3273, 0x36);
	NT99230_write_cmos_sensor(0x3274, 0x4B);
	NT99230_write_cmos_sensor(0x3275, 0x5D);
	NT99230_write_cmos_sensor(0x3276, 0x7E);
	NT99230_write_cmos_sensor(0x3277, 0x98);
	NT99230_write_cmos_sensor(0x3278, 0xAC);
	NT99230_write_cmos_sensor(0x3279, 0xBD);
	NT99230_write_cmos_sensor(0x327A, 0xD4);
	NT99230_write_cmos_sensor(0x327B, 0xE5);
	NT99230_write_cmos_sensor(0x327C, 0xF0);
	NT99230_write_cmos_sensor(0x327D, 0xF4);  
	NT99230_write_cmos_sensor(0x327E, 0xF8);	
	NT99230_write_cmos_sensor(0x3700, 0x04);  
	NT99230_write_cmos_sensor(0x3701, 0x0F);
	NT99230_write_cmos_sensor(0x3702, 0x1A);
	NT99230_write_cmos_sensor(0x3703, 0x32);
	NT99230_write_cmos_sensor(0x3704, 0x42);
	NT99230_write_cmos_sensor(0x3705, 0x51);
	NT99230_write_cmos_sensor(0x3706, 0x6B);
	NT99230_write_cmos_sensor(0x3707, 0x81);
	NT99230_write_cmos_sensor(0x3708, 0x94);
	NT99230_write_cmos_sensor(0x3709, 0xA6);
	NT99230_write_cmos_sensor(0x370A, 0xC2);
	NT99230_write_cmos_sensor(0x370B, 0xD4);
	NT99230_write_cmos_sensor(0x370C, 0xE0);
	NT99230_write_cmos_sensor(0x370D, 0xE7); 
	NT99230_write_cmos_sensor(0x370E, 0xEF);  
	NT99230_write_cmos_sensor(0x3710, 0x02);
	NT99230_write_cmos_sensor(0x3714, 0x04);
	NT99230_write_cmos_sensor(0x3800, 0x00);
	NT99230_write_cmos_sensor(0x3302, 0x00); 
	NT99230_write_cmos_sensor(0x3303, 0x4C);
	NT99230_write_cmos_sensor(0x3304, 0x00);
	NT99230_write_cmos_sensor(0x3305, 0x96);
	NT99230_write_cmos_sensor(0x3306, 0x00);
	NT99230_write_cmos_sensor(0x3307, 0x1D);
	NT99230_write_cmos_sensor(0x3308, 0x07);
	NT99230_write_cmos_sensor(0x3309, 0xC7);
	NT99230_write_cmos_sensor(0x330A, 0x06);
	NT99230_write_cmos_sensor(0x330B, 0xB6);
	NT99230_write_cmos_sensor(0x330C, 0x01);
	NT99230_write_cmos_sensor(0x330D, 0x83);
	NT99230_write_cmos_sensor(0x330E, 0x01);
	NT99230_write_cmos_sensor(0x330F, 0x1C);
	NT99230_write_cmos_sensor(0x3310, 0x06);
	NT99230_write_cmos_sensor(0x3311, 0xC2);
	NT99230_write_cmos_sensor(0x3312, 0x00);
	NT99230_write_cmos_sensor(0x3313, 0x22);
	NT99230_write_cmos_sensor(0x3250, 0x37);	
	NT99230_write_cmos_sensor(0x3251, 0x1F);
	NT99230_write_cmos_sensor(0x3252, 0x39);
	NT99230_write_cmos_sensor(0x3253, 0x21);
	NT99230_write_cmos_sensor(0x3254, 0x3B);
	NT99230_write_cmos_sensor(0x3255, 0x23);
	NT99230_write_cmos_sensor(0x3256, 0x29);
	NT99230_write_cmos_sensor(0x3257, 0x16);
	NT99230_write_cmos_sensor(0x3258, 0x4F);
	NT99230_write_cmos_sensor(0x3259, 0x35);
	NT99230_write_cmos_sensor(0x325A, 0x29);
	NT99230_write_cmos_sensor(0x325B, 0x14);
	NT99230_write_cmos_sensor(0x325C, 0x70);                 
	NT99230_write_cmos_sensor(0x325D, 0x08); 
	NT99230_write_cmos_sensor(0x3290, 0x50);
	NT99230_write_cmos_sensor(0x3292, 0x50);
	NT99230_write_cmos_sensor(0x3297, 0x03);                                              
	NT99230_write_cmos_sensor(0x32B8, 0x07);                                                
	NT99230_write_cmos_sensor(0x32B9, 0x07); 
	NT99230_write_cmos_sensor(0x32B0, 0x55);
	NT99230_write_cmos_sensor(0x32B1, 0xAA);
	NT99230_write_cmos_sensor(0x32B2, 0x14); 
	NT99230_write_cmos_sensor(0x32B3, 0xA0); 
	NT99230_write_cmos_sensor(0x32B4, 0x00); 
	NT99230_write_cmos_sensor(0x32BC, 0x44);                           
	NT99230_write_cmos_sensor(0x32BD, 0x04);                
	NT99230_write_cmos_sensor(0x32BE, 0x04); 
	NT99230_write_cmos_sensor(0x32CB, 0x18);              
	NT99230_write_cmos_sensor(0x32CC, 0x70);                
	NT99230_write_cmos_sensor(0x32CD, 0xA0);                                
	NT99230_write_cmos_sensor(0x3326, 0x10); 
	NT99230_write_cmos_sensor(0x3327, 0x01);  
	NT99230_write_cmos_sensor(0x3332, 0x80);
	NT99230_write_cmos_sensor(0x334A, 0x00);
	NT99230_write_cmos_sensor(0x334B, 0x7F);
	NT99230_write_cmos_sensor(0x334C, 0x1F);
	NT99230_write_cmos_sensor(0x335B, 0xD0);                
	NT99230_write_cmos_sensor(0x3360, 0x0A);                              
	NT99230_write_cmos_sensor(0x3361, 0x1F);                
	NT99230_write_cmos_sensor(0x3362, 0x2F); 
	NT99230_write_cmos_sensor(0x3365, 0x08);                
	NT99230_write_cmos_sensor(0x3366, 0x0C);                
	NT99230_write_cmos_sensor(0x3367, 0x10);                
	NT99230_write_cmos_sensor(0x3368, 0x40);                
	NT99230_write_cmos_sensor(0x3369, 0x38);                
	NT99230_write_cmos_sensor(0x336B, 0x28);                
	NT99230_write_cmos_sensor(0x336D, 0x20);                
	NT99230_write_cmos_sensor(0x336E, 0x12);                
	NT99230_write_cmos_sensor(0x3370, 0x08); 
	NT99230_write_cmos_sensor(0x3371, 0x3F);                                     
	NT99230_write_cmos_sensor(0x3372, 0x3F); 
	NT99230_write_cmos_sensor(0x3374, 0x3F); 
	NT99230_write_cmos_sensor(0x3375, 0x10); 
	NT99230_write_cmos_sensor(0x3376, 0x10); 
	NT99230_write_cmos_sensor(0x3378, 0x10); 
	NT99230_write_cmos_sensor(0x3379, 0x04); 
	NT99230_write_cmos_sensor(0x337A, 0x06); 
	NT99230_write_cmos_sensor(0x337C, 0x08); 
	NT99230_write_cmos_sensor(0x33A0, 0x40); 
	NT99230_write_cmos_sensor(0x33A1, 0x68); 
	NT99230_write_cmos_sensor(0x33A2, 0x10);                
	NT99230_write_cmos_sensor(0x33A3, 0x40);                
	NT99230_write_cmos_sensor(0x33A4, 0x00);                
	NT99230_write_cmos_sensor(0x33A5, 0x8C);                
	NT99230_write_cmos_sensor(0x33A6, 0x02);                
	NT99230_write_cmos_sensor(0x33A7, 0x00);                                                 
	NT99230_write_cmos_sensor(0x33A9, 0x04);                               
	NT99230_write_cmos_sensor(0x33AA, 0x08);                
	NT99230_write_cmos_sensor(0x33AC, 0x0C);                
	NT99230_write_cmos_sensor(0x33AD, 0x03);                
	NT99230_write_cmos_sensor(0x33AE, 0x05);                
	NT99230_write_cmos_sensor(0x33B0, 0x08); 
	NT99230_write_cmos_sensor(0x33B1, 0x00);                    
	NT99230_write_cmos_sensor(0x33B4, 0x48); 
	NT99230_write_cmos_sensor(0x33B5, 0xA4); 
	NT99230_write_cmos_sensor(0x33BA, 0x03); 
	NT99230_write_cmos_sensor(0x33BB, 0x07); 
	NT99230_write_cmos_sensor(0x33C0, 0x01); 
	NT99230_write_cmos_sensor(0x33C6, 0x03); 
	NT99230_write_cmos_sensor(0x33C7, 0x43); 
	NT99230_write_cmos_sensor(0x33C8, 0x33); 
	NT99230_write_cmos_sensor(0x33C9, 0x0A); 
	NT99230_write_cmos_sensor(0x3363, 0x31); 
	NT99230_write_cmos_sensor(0x3364, 0x0B);
	NT99230_write_cmos_sensor(0x333F, 0x0F);
	NT99230_write_cmos_sensor(0x3012, 0x03);
	NT99230_write_cmos_sensor(0x3013, 0x24);
	NT99230_write_cmos_sensor(0x3290, 0x82);
	NT99230_write_cmos_sensor(0x3292, 0x7A);
	NT99230_write_cmos_sensor(0x3060, 0x04);
} 

/*************************************************************************
 * FUNCTION
 *	NT99230Open
 *
 * DESCRIPTION
 *	This function initialize the registers of CMOS sensor
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99230Open(void)
{
	kal_uint16 sensor_id=0;
	int i;
	printk(KERN_EMERG"NT99230 Open File Start\n");
	SENSORDB("@@@SP:Ronlus NT99230Open\r\n");
	// check if sensor ID correct
	for(i = 0; i < 3; i++)
	{

		sensor_id = NT99230_read_cmos_sensor(0x3000);
		sensor_id = (sensor_id << 8) | NT99230_read_cmos_sensor(0x3001);
		SENSORDB("@@@SP:Ronlus NT99230 Sensor id = %x\n", sensor_id);

		if (sensor_id == NT99230_SENSOR_ID)
		{
			break;
		}
	}
	//mdelay(50);
	if(sensor_id != NT99230_SENSOR_ID)
	{
		SENSORDB("@@@SP:NT99230 Sensor id read failed, ID = %x\n", sensor_id);
		return ERROR_SENSOR_CONNECT_FAIL;
	}
	SENSORDB("@@@SP:NT99230 Sensor id read, ID = %x\n", sensor_id);

#ifdef DEBUG_SENSOR_NT99230  //gepeiwei   120903
	struct file *fp; 
	mm_segment_t fs; 
	loff_t pos = 0; 
	//static char buf[10*1024] ;

	printk("NT99230 Open File Start\n");
	fp = filp_open("/storage/sdcard1/nt99230_sd.dat", O_RDONLY , 0); 
	if (IS_ERR(fp)) 
	{ 
		printk("open file error 0\n");
		fp = filp_open("/storage/sdcard0/nt99230_sd.dat", O_RDONLY , 0); 
		if (IS_ERR(fp)) { 
			fromsd = 0;   
			printk("open file error 1\n");
		}
		else{
			printk("open file success 1\n");
			fromsd = 1;
			filp_close(fp, NULL); 
		}
	} 
	else 
	{
		printk("open file success 0\n");
		fromsd = 1;
		printk("open file ok\n");

		//NT99230_Initialize_from_T_Flash();


		filp_close(fp, NULL); 
	}

	if(fromsd == 1)//Ƿ\u017dSDȡ//gepeiwei   120903
	{
		printk("________________from t!\n");
		NT99230_Initialize_from_T_Flash();//\u017dSD\u0161ȡҪ
	}
	else
	{
		NT99230_Sensor_Init();
		NT99230_Write_More_Registers();//added for FAE to debug
	}
#else  
	//RETAILMSG(1, (TEXT("Sensor Read ID OK \r\n")));
	// initail sequence write in
	NT99230_Sensor_Init();
	NT99230_Write_More_Registers();//added for FAE to debut
#endif	
return ERROR_NONE;
} /* NT99230Open */


/*************************************************************************
 * FUNCTION
 *	NT99230Close
 *
 * DESCRIPTION
 *	This function is to turn off sensor module power.
 *
 * PARAMETERS
 *	None
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99230Close(void)
{
	SENSORDB("@@@SP:NT99230Close\r\n");
	return ERROR_NONE;
} /* NT99230Close */

void NT99230_AfterSnapshot(void)
{

}  


void NT99230_BeforeSnapshot(void)
{
//	return 0;  
} 

/*************************************************************************
 * FUNCTION
 * NT99230Preview
 *
 * DESCRIPTION
 *	This function start the sensor preview.
 *
 * PARAMETERS
 *	*image_window : address pointer of pixel numbers in one period of HSYNC
 *  *sensor_config_data : address pointer of line numbers in one period of VSYNC
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99230Preview(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
		MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)
{
	//kal_uint32 iTemp;
	//kal_uint16 iStartX = 0, iStartY = 1;
	printk("@@@SP:Ronlus NT99230Preview fun start\r\n");

    NT99230_MODE_CAPTURE = KAL_FALSE;
    NT99230_gPVmode = KAL_TRUE;

    #if PREVIEW_USE_VGA
        NT99230_config_window(WINDOW_SIZE_VGA);
    #elif PREVIEW_USE_SVGA
        NT99230_config_window(WINDOW_SIZE_SVGA);
    #elif PREVIEW_USE_720P
        NT99230_config_window(WINDOW_SIZE_720P);
	 #elif PREVIEW_USE_1080P
        NT99230_config_window(PREVIEW_USE_1080P);
    #else
        NT99230_config_window(PREVIEW_USE_1080P);
    #endif
    
    if (setshutter) 
    {
	    ///NT99230_AfterSnapshot(); 
    }
	setshutter = KAL_FALSE;
	if(sensor_config_data->SensorOperationMode == MSDK_SENSOR_OPERATION_MODE_VIDEO)		// MPEG4 Encode Mode
	{
		printk(KERN_EMERG"@@@SP:Ronlus video preview\r\n");
		NT99230_MPEG4_encode_mode = KAL_TRUE;
	}
	else
	{
		printk(KERN_EMERG"@@@SP:Ronlus capture preview\r\n");
		NT99230_MPEG4_encode_mode = KAL_FALSE;
	}
	//NT99230_config_window(WINDOW_SIZE_SVGA);//add zch test(use this for SVGA)
    image_window->GrabStartX= 1;
    image_window->GrabStartY= 1;
    image_window->ExposureWindowWidth = IMAGE_SENSOR_PV_WIDTH;//800-16; //1280-16; //1600-32 ; //800-16; 
    image_window->ExposureWindowHeight = IMAGE_SENSOR_PV_HEIGHT;//600-12;//720-12; //1200-24; //600-12;

    NT99230_DELAY_AFTER_PREVIEW = 1;
	// copy sensor_config_data
	memcpy(&NT99230SensorConfigData, sensor_config_data, sizeof(MSDK_SENSOR_CONFIG_STRUCT));//rotation
	printk(KERN_EMERG"@@@SP:Ronlus NT99230Preview fun end\r\n");
	//Sleep(300);
	printk(KERN_EMERG"@@@SP:555555\r\n");
	return ERROR_NONE;
} /* NT99230Preview */


/*************************************************************************
 * FUNCTION
 *	NT99230Capture
 *
 * DESCRIPTION
 *	This function setup the CMOS sensor in capture MY_OUTPUT mode
 *
 * PARAMETERS
 *
 * RETURNS
 *	None
 *
 * GLOBALS AFFECTED
 *
 *************************************************************************/
UINT32 NT99230Capture(MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *image_window,
		MSDK_SENSOR_CONFIG_STRUCT *sensor_config_data)

{
	SENSORDB("Ronlus NT99230Capture fun start\r\n");
	NT99230_MODE_CAPTURE=KAL_TRUE;

    // config: UXGA for capture
    NT99230_config_window(PREVIEW_USE_1080P);
	
     if ((image_window->ImageTargetWidth<=640-16)&&
        (image_window->ImageTargetHeight<=480-12))
		{    /* Less than PV Mode */
			image_window->GrabStartX = 0;
			image_window->GrabStartY = 1;
			image_window->ExposureWindowWidth= 640-16;
			image_window->ExposureWindowHeight = 480-12;
		}
     else

	{
//		kal_uint32 shutter, cap_dummy_pixels = 0; 
		if(!setshutter)
		{
			//// NT99230_BeforeSnapshot();
			image_window->GrabStartX = 1;//iStartX;
			image_window->GrabStartY = 1;//iStartY;
			image_window->ExposureWindowWidth = IMAGE_SENSOR_FULL_WIDTH; //16 ;
			image_window->ExposureWindowHeight = IMAGE_SENSOR_FULL_HEIGHT; //12;
		}
		else
		{
			image_window->GrabStartX = 1;
			image_window->GrabStartY = 1;
			image_window->ExposureWindowWidth = IMAGE_SENSOR_FULL_WIDTH; //16 ;
			image_window->ExposureWindowHeight = IMAGE_SENSOR_FULL_HEIGHT; //12;
		}

		setshutter = KAL_TRUE;
		
	}

	 NT99230_DELAY_AFTER_PREVIEW = 2;
	// copy sensor_config_data
	memcpy(&NT99230SensorConfigData, sensor_config_data, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
	SENSORDB("Ronlus NT99230Capture fun end\r\n");
	return ERROR_NONE;
} /* NT99230_Capture() */




UINT32 NT99230GetResolution(MSDK_SENSOR_RESOLUTION_INFO_STRUCT *pSensorResolution)
{
	SENSORDB("Ronlus NT99230GetResolution\r\n");
    pSensorResolution->SensorFullWidth = IMAGE_SENSOR_FULL_WIDTH; //1600-96; //-16-32;
    pSensorResolution->SensorFullHeight = IMAGE_SENSOR_FULL_HEIGHT; //1200-72; //12-24;
    pSensorResolution->SensorPreviewWidth = IMAGE_SENSOR_PV_WIDTH; //1600-64; //800-16-8;//IMAGE_SENSOR_SVGA_WIDTH;(user this for SVGA)
    pSensorResolution->SensorPreviewHeight = IMAGE_SENSOR_PV_HEIGHT; //1200-48; //600-12-6;//IMAGE_SIMAGE_SENSOR_SVGA_HEIGHT;(use this for SVGA)
	pSensorResolution->SensorVideoWidth = IMAGE_SENSOR_FULL_WIDTH; //1600-64;  //800-16-8;
	pSensorResolution->SensorVideoHeight = IMAGE_SENSOR_FULL_HEIGHT; //1200-48; //600-12-6;
	
    pSensorResolution->SensorHighSpeedVideoWidth = IMAGE_SENSOR_PV_WIDTH;
    pSensorResolution->SensorHighSpeedVideoHeight = IMAGE_SENSOR_PV_HEIGHT;
    
    pSensorResolution->SensorSlimVideoWidth = IMAGE_SENSOR_PV_WIDTH;
    pSensorResolution->SensorSlimVideoHeight = IMAGE_SENSOR_PV_HEIGHT;  
	return ERROR_NONE;
} /* NT99230GetResolution() */ 


UINT32 NT99230GetInfo(MSDK_SCENARIO_ID_ENUM ScenarioId,
		MSDK_SENSOR_INFO_STRUCT *pSensorInfo,
		MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData)
{
	SENSORDB("@@@SP:Ronlus NT99230GetInfo\r\n");
    pSensorInfo->SensorPreviewResolutionX = IMAGE_SENSOR_PV_WIDTH; //1600-64; //800-16-8;//IMAGE_SENSOR_PV_WIDTH(use this for SVGA);
    pSensorInfo->SensorPreviewResolutionY = IMAGE_SENSOR_PV_HEIGHT; //1200-48; //600-12-6;//IMAGE_SENSOR_PV_HEIGHT(user this for SVGA);
    pSensorInfo->SensorFullResolutionX = IMAGE_SENSOR_FULL_WIDTH; //1600-96; //-16-32;
    pSensorInfo->SensorFullResolutionY = IMAGE_SENSOR_FULL_HEIGHT; //1200-72; //-12-24;

	pSensorInfo->SensorCameraPreviewFrameRate=30;  
	pSensorInfo->SensorVideoFrameRate=30;
	pSensorInfo->SensorStillCaptureFrameRate=10;
	pSensorInfo->SensorWebCamCaptureFrameRate=30; 
	pSensorInfo->SensorResetActiveHigh=TRUE;
	pSensorInfo->SensorResetDelayCount=1;
	pSensorInfo->SensorOutputDataFormat=SENSOR_OUTPUT_FORMAT_UYVY;
	pSensorInfo->SensorClockPolarity=SENSOR_CLOCK_POLARITY_LOW;
	pSensorInfo->SensorClockFallingPolarity=SENSOR_CLOCK_POLARITY_LOW;
	
	pSensorInfo->SensorHsyncPolarity =SENSOR_CLOCK_POLARITY_LOW;
	pSensorInfo->SensorVsyncPolarity = SENSOR_CLOCK_POLARITY_LOW;
	pSensorInfo->SensorInterruptDelayLines = 2;
	pSensorInfo->SensroInterfaceType=SENSOR_INTERFACE_TYPE_MIPI;
    pSensorInfo->SensorMIPILaneNumber = SENSOR_MIPI_2_LANE;
    //pSensorInfo->MIPIDataLowPwr2HighSpeedTermDelayCount = 0;
    //pSensorInfo->MIPIDataLowPwr2HighSpeedSettleDelayCount = 4;
    //pSensorInfo->MIPICLKLowPwr2HighSpeedTermDelayCount = 0;

	
	pSensorInfo->MIPIDataLowPwr2HighSpeedTermDelayCount = 0;
	pSensorInfo->MIPIDataLowPwr2HighSpeedSettleDelayCount = 14; // 130;  // 13;  // 17; // 4; // 14;    // 4; 
	pSensorInfo->MIPICLKLowPwr2HighSpeedTermDelayCount = 0;	
	pSensorInfo->MIPIsensorType = MIPI_OPHY_NCSI2; 
	pSensorInfo->SensorWidthSampling = 0;   // 0 is default 1x
    pSensorInfo->SensorHightSampling = 0;   // 0 is default 1x
    pSensorInfo->SensorPacketECCOrder = 1;

	
	pSensorInfo->CaptureDelayFrame = 1;
	pSensorInfo->PreviewDelayFrame = 1;
	pSensorInfo->VideoDelayFrame = 4;
	pSensorInfo->SensorMasterClockSwitch = 0;
	pSensorInfo->SensorDrivingCurrent = ISP_DRIVING_8MA;///ISP_DRIVING_2MA; //modify by sp_yjp,20130813
	
	pSensorInfo->HighSpeedVideoDelayFrame = 4;
	pSensorInfo->SlimVideoDelayFrame = 4;
 	pSensorInfo->SensorModeNum = 5;


	switch (ScenarioId)
	{
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			//case MSDK_SCENARIO_ID_VIDEO_CAPTURE_MPEG4:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount=	3;
			pSensorInfo->SensorClockRisingCount= 0;
			pSensorInfo->SensorClockFallingCount= 2;
			pSensorInfo->SensorPixelClockCount= 3;
			pSensorInfo->SensorDataLatchCount= 2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;

			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			//  case MSDK_SCENARIO_ID_CAMERA_CAPTURE_MEM:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount= 3;
			pSensorInfo->SensorClockRisingCount=0;
			pSensorInfo->SensorClockFallingCount=2;
			pSensorInfo->SensorPixelClockCount=3;
			pSensorInfo->SensorDataLatchCount=2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;
			break;
		default:
			pSensorInfo->SensorClockFreq=24;
			pSensorInfo->SensorClockDividCount= 3;
			pSensorInfo->SensorClockRisingCount=0;
			pSensorInfo->SensorClockFallingCount=2;
			pSensorInfo->SensorPixelClockCount=3;
			pSensorInfo->SensorDataLatchCount=2;
        	pSensorInfo->SensorGrabStartX = 1;
        	pSensorInfo->SensorGrabStartY = 1;
			break;
	}
	NT99230PixelClockDivider=pSensorInfo->SensorPixelClockCount;
	memcpy(pSensorConfigData, &NT99230SensorConfigData, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
	return ERROR_NONE;
} /* NT99230GetInfo() */


UINT32 NT99230Control(MSDK_SCENARIO_ID_ENUM ScenarioId, MSDK_SENSOR_EXPOSURE_WINDOW_STRUCT *pImageWindow,
		MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData)
{
	pr_err("@@@SP:Ronlus NT99230Control id %d\r\n",ScenarioId);
	
	switch (ScenarioId)
	{
		case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
		case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
			//case MSDK_SCENARIO_ID_VIDEO_CAPTURE_MPEG4:
			NT99230Preview(pImageWindow, pSensorConfigData);
			break;
		case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
			//case MSDK_SCENARIO_ID_CAMERA_CAPTURE_MEM:
			NT99230Capture(pImageWindow, pSensorConfigData);
			break;
		default:
            SENSORDB("Error ScenarioId setting");
            NT99230Preview(pImageWindow, pSensorConfigData);
            return ERROR_INVALID_SCENARIO_ID;
	}


	return TRUE;
}	/* NT99230Control() */

BOOL NT99230_set_param_wb(UINT16 para)
{
	SENSORDB("Ronlus NT99230_set_param_wb para = %d\r\n",para);
	switch (para)
	{
		case AWB_MODE_OFF:
			break; 					
		case AWB_MODE_AUTO:
			break;
		case AWB_MODE_CLOUDY_DAYLIGHT: //cloudy

			break;
		case AWB_MODE_DAYLIGHT: //sunny

			break;
		case AWB_MODE_INCANDESCENT: //office

			break;
		case AWB_MODE_TUNGSTEN: //home

			break;
		case AWB_MODE_FLUORESCENT:

		default:
			return FALSE;
	}  
	return TRUE;
} /* NT99230_set_param_wb */


BOOL NT99230_set_param_effect(UINT16 para)
{
	SENSORDB("Ronlus NT99230_set_param_effect\r\n");
	switch (para)
	{
		case MEFFECT_OFF:

			break;
		case MEFFECT_SEPIA:

			break;
		case MEFFECT_NEGATIVE://----datasheet

			break;
		case MEFFECT_SEPIAGREEN://----datasheet aqua

			break;
		case MEFFECT_SEPIABLUE:

			break;
		case MEFFECT_MONO: //----datasheet black & white

			break;
		default:
			return FALSE;
	}
	return TRUE;
} /* NT99230_set_param_effect */

//UINT8 index = 1;
BOOL NT99230_set_param_banding(UINT16 para)
{
	//UINT16 buffer = 0;
	UINT8 index = 1;
	SENSORDB("Ronlus NT99230_set_param_banding para = %d ---- index = %d\r\n",para,index); 
	SENSORDB("Ronlus NT99230_set_param_banding ---- NT99230_MPEG4_encode_mode = %d\r\n",NT99230_MPEG4_encode_mode);
	switch (para)
	{
		case AE_FLICKER_MODE_50HZ:

			NT99230_CAM_BANDING_50HZ = KAL_TRUE;
			break;
		case AE_FLICKER_MODE_60HZ:

			NT99230_CAM_BANDING_50HZ = KAL_FALSE;
			break;
		default:
			//NT99230_write_cmos_sensor(0xfd,0x00);//ronlus test
			//buffer = NT99230_read_cmos_sensor(0x35);
			return FALSE;
	}
#if 0/*Superpix Ronlus some vertical line when switching*/
	NT99230_config_window(WINDOW_SIZE_SVGA);
	NT99230_write_cmos_sensor(0xfd,0x00);
	NT99230_write_cmos_sensor(0x35,0x00);
#endif
	return TRUE;
} /* NT99230_set_param_banding */


BOOL NT99230_set_param_exposure(UINT16 para)
{
	SENSORDB("Ronlus NT99230_set_param_exposure\r\n");
	switch (para)
	{
		case AE_EV_COMP_n30:              /* EV -2 */

			break;
		case AE_EV_COMP_n20:              /* EV -1.5 */

			break;
		case AE_EV_COMP_n10:              /* EV -1 */

			break;
		case AE_EV_COMP_n03:              
			break;  
		
		case AE_EV_COMP_00:                /* EV 0 */

			break;
		case AE_EV_COMP_10:              /* EV +0.5 */

			break;
		case AE_EV_COMP_20:              /* EV +1 */

			break;
		case AE_EV_COMP_30:              /* EV +1.5 */

			break;
		case AE_EV_COMP_13:             
			break;  
		default:
			return FALSE;
	}
	return TRUE;

} /* NT99230_set_param_exposure */


UINT32 NT99230YUVSensorSetting(FEATURE_ID iCmd, UINT16 iPara)
{

#ifdef DEBUG_SENSOR_NT99230
	return TRUE;
#endif
	//SENSORDB("Ronlus NT99230YUVSensorSetting\r\n");
	switch (iCmd) 
	{
		case FID_SCENE_MODE:	
			#if 0 //del by sp_yjp for test,20130822
			if (iPara == SCENE_MODE_OFF)
			{
				NT99230_night_mode(0); 
			}
			else if (iPara == SCENE_MODE_NIGHTSCENE)
			{
				NT99230_night_mode(1); 
			}
			#endif
			break; 	    
		case FID_AWB_MODE:
			NT99230_set_param_wb(iPara);
			break;
		case FID_COLOR_EFFECT:
			NT99230_set_param_effect(iPara);
			break;
		case FID_AE_EV:
			NT99230_set_param_exposure(iPara);
			break;
		case FID_AE_FLICKER:
			NT99230_set_param_banding(iPara);
			NT99230_night_mode(NT99230_CAM_Nightmode); 
			break;
		default:
			break;
	}
	return TRUE;
} /* NT99230YUVSensorSetting */

UINT32 nt99230_get_sensor_id(UINT32 *sensorID) 
{
	volatile signed char i;
	kal_uint16 sensor_id=0;
	printk("@@@SP:xieyang NT99230GetSensorID ");
	///SENSORDB("@@@SP:xieyang in GPIO_CAMERA_CMPDN_PIN=%d,GPIO_CAMERA_CMPDN1_PIN=%d"
	///		, mt_get_gpio_out(GPIO_CAMERA_CMPDN_PIN),mt_get_gpio_out(GPIO_CAMERA_CMPDN1_PIN));
	for(i=0;i<10;i++)
	{
		sensor_id = NT99230_read_cmos_sensor(0x3000);
		sensor_id = (sensor_id << 8) | NT99230_read_cmos_sensor(0x3001);
		if(sensor_id == NT99230_SENSOR_ID)
			break;
	}
	printk("@@@SP:NT99230GetSensorID=%#x\r\n ",sensor_id);

	while(0)	//add by sp_yjp for test,20130813
	{
		mdelay(50);	
		sensor_id = NT99230_read_cmos_sensor(0x3000);
		printk("@@@SP:DeadReadNT99230ID=%#x\r\n ",sensor_id);
		if(sensor_id == NT99230_SENSOR_ID)
			break;
	}
	
	if(sensor_id != NT99230_SENSOR_ID)
	{
		printk("@@@SP:>>>>>>>>>>NT99230 read ID error<<<<<<<<<<<");			
		*sensorID = 0xFFFFFFFF;
		return ERROR_SENSOR_CONNECT_FAIL;
	}
	*sensorID = sensor_id;
	return ERROR_NONE;
	
}


UINT32 NT99230_SetTestPatternMode(kal_bool bEnable)
{
 //   kal_uint16 temp;

    SENSORDB("bEnable: %d", bEnable);

    if(bEnable) 
	{
	}
	else        
	{
	}
    return ERROR_NONE;
}

//#define NT99230_TEST_PATTERN_CHECKSUM (0x4d1d696d)
//#define NT99230_TEST_PATTERN_CHECKSUM (0x96f31f11)
#define NT99230_TEST_PATTERN_CHECKSUM (0x3f4b965a)


UINT32 NT99230FeatureControl(MSDK_SENSOR_FEATURE_ENUM FeatureId,
		UINT8 *pFeaturePara,UINT32 *pFeatureParaLen)
{
	UINT16 *pFeatureReturnPara16=(UINT16 *) pFeaturePara;
//	UINT16 *pFeatureData16=(UINT16 *) pFeaturePara;
	UINT32 *pFeatureReturnPara32=(UINT32 *) pFeaturePara;
	UINT32 *pFeatureData32=(UINT32 *) pFeaturePara;
	
    unsigned long long *pFeatureData=(unsigned long long *) pFeaturePara;
//  unsigned long long *pFeatureReturnPara=(unsigned long long *) pFeaturePara;
//	UINT32 NT99230SensorRegNumber;
//	UINT32 i;

	SENSOR_WINSIZE_INFO_STRUCT *wininfo;
	
	
	MSDK_SENSOR_CONFIG_STRUCT *pSensorConfigData=(MSDK_SENSOR_CONFIG_STRUCT *) pFeaturePara;
	MSDK_SENSOR_REG_INFO_STRUCT *pSensorRegData=(MSDK_SENSOR_REG_INFO_STRUCT *) pFeaturePara;

	//pr_debug("@@@SP:Ronlus NT99230FeatureControl.---FeatureId = %d\r\n",FeatureId);
	RETAILMSG(1, (_T("gaiyang NT99230FeatureControl FeatureId=%d\r\n"), FeatureId));

	switch (FeatureId)
	{
		case SENSOR_FEATURE_GET_RESOLUTION:
			*pFeatureReturnPara16++=N1080P_PERIOD_PIXEL_NUMS;
			*pFeatureReturnPara16=N1080P_PERIOD_LINE_NUMS;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_GET_PERIOD:
			*pFeatureReturnPara16++=(N1080P_PERIOD_PIXEL_NUMS)+NT99230_dummy_pixels;
			*pFeatureReturnPara16=(N1080P_PERIOD_PIXEL_NUMS)+NT99230_dummy_lines;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_GET_PIXEL_CLOCK_FREQ:
			*pFeatureReturnPara32 = NT99230_g_fPV_PCLK;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_SET_ESHUTTER:
			break;
		case SENSOR_FEATURE_SET_NIGHTMODE:
#ifndef DEBUG_SENSOR_NT99230		
			NT99230_night_mode((BOOL) *pFeatureData);
#endif
			break;
		case SENSOR_FEATURE_SET_GAIN:
		case SENSOR_FEATURE_SET_FLASHLIGHT:
			break;
		case SENSOR_FEATURE_SET_ISP_MASTER_CLOCK_FREQ:
			NT99230_isp_master_clock=*pFeatureData32;
			break;
		case SENSOR_FEATURE_SET_REGISTER:
			NT99230_write_cmos_sensor(pSensorRegData->RegAddr, pSensorRegData->RegData,0x36);
			break;
		case SENSOR_FEATURE_GET_REGISTER:
			pSensorRegData->RegData = NT99230_read_cmos_sensor(pSensorRegData->RegAddr);
			break;
		case SENSOR_FEATURE_GET_CONFIG_PARA:
			memcpy(pSensorConfigData, &NT99230SensorConfigData, sizeof(MSDK_SENSOR_CONFIG_STRUCT));
			*pFeatureParaLen=sizeof(MSDK_SENSOR_CONFIG_STRUCT);
			break;
		case SENSOR_FEATURE_SET_CCT_REGISTER:
		case SENSOR_FEATURE_GET_CCT_REGISTER:
		case SENSOR_FEATURE_SET_ENG_REGISTER:
		case SENSOR_FEATURE_GET_ENG_REGISTER:
		case SENSOR_FEATURE_GET_REGISTER_DEFAULT:
		case SENSOR_FEATURE_CAMERA_PARA_TO_SENSOR:
		case SENSOR_FEATURE_SENSOR_TO_CAMERA_PARA:
		case SENSOR_FEATURE_GET_GROUP_COUNT:
		case SENSOR_FEATURE_GET_GROUP_INFO:
		case SENSOR_FEATURE_GET_ITEM_INFO:
		case SENSOR_FEATURE_SET_ITEM_INFO:
		case SENSOR_FEATURE_GET_ENG_INFO:
			break;
		case SENSOR_FEATURE_CHECK_SENSOR_ID:
			nt99230_get_sensor_id(pFeatureReturnPara32); 
			break; 
		case SENSOR_FEATURE_GET_LENS_DRIVER_ID:
			// get the lens driver ID from EEPROM or just return LENS_DRIVER_ID_DO_NOT_CARE
			// if EEPROM does not exist in camera module.
			*pFeatureReturnPara32=LENS_DRIVER_ID_DO_NOT_CARE;
			*pFeatureParaLen=4;
			break;
		case SENSOR_FEATURE_SET_YUV_CMD:
			NT99230YUVSensorSetting((FEATURE_ID)*pFeatureData, *(pFeatureData+1));//test ,del by sp_yjp,20130813
			break;
		case SENSOR_FEATURE_SET_TEST_PATTERN:
        	NT99230_SetTestPatternMode((BOOL)*pFeatureData);
       		break;
    	case SENSOR_FEATURE_GET_TEST_PATTERN_CHECKSUM_VALUE://for factory mode auto testing             
        	*pFeatureReturnPara32= NT99230_TEST_PATTERN_CHECKSUM;
        	*pFeatureParaLen=4; 
			break;
		case SENSOR_FEATURE_GET_CROP_INFO:
            //LOG_INF("SENSOR_FEATURE_GET_CROP_INFO scenarioId:%d\n", *pFeatureData);
            // For pointer address in user space is 32-bit, need use compat_ptr to convert it to 64-bit in kernel space    

            wininfo = (SENSOR_WINSIZE_INFO_STRUCT *)(uintptr_t)(*(pFeatureData+1));

            switch (*pFeatureData32) {
                case MSDK_SCENARIO_ID_CAMERA_CAPTURE_JPEG:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[1],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_VIDEO_PREVIEW:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[2],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_HIGH_SPEED_VIDEO:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[3],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_SLIM_VIDEO:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[4],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
                case MSDK_SCENARIO_ID_CAMERA_PREVIEW:
                default:
                    memcpy((void *)wininfo,(void *)&imgsensor_winsize_info[0],sizeof(SENSOR_WINSIZE_INFO_STRUCT));
                    break;
            }
			break;
			
		default:
			break;
	}
	return ERROR_NONE;
}	/* NT99230FeatureControl() */


SENSOR_FUNCTION_STRUCT	SensorFuncNT99230MIPIYUV=
{
	NT99230Open,
	NT99230GetInfo,
	NT99230GetResolution,
	NT99230FeatureControl,
	NT99230Control,
	NT99230Close
};


UINT32 NT99230_MIPI_YUV_SensorInit(PSENSOR_FUNCTION_STRUCT *pfFunc)
{
	/* To Do : Check Sensor status here */	
	if (pfFunc!=NULL)
	{
		SENSORDB("@@@SP:NT99230_YUV_SensorInit\r\n");
		*pfFunc=&SensorFuncNT99230MIPIYUV;
	}
	return ERROR_NONE;
} /* SensorInit() */
