#include <linux/string.h>
#include <linux/gpio.h>
#include <mt-plat/mt_gpio.h>
#include "lcm_drv.h"
#include <linux/leds.h>

// ---------------------------------------------------------------------------
//  Local Constants
// ---------------------------------------------------------------------------



// ---------------------------------------------------------------------------
//  Local Variables
// ---------------------------------------------------------------------------

static LCM_UTIL_FUNCS lcm_util = {0};

#define SET_RESET_PIN(v)    (lcm_util.set_reset_pin((v)))

#define UDELAY(n) (lcm_util.udelay(n))
#define MDELAY(n) (lcm_util.mdelay(n))


// ---------------------------------------------------------------------------
//  LCM Driver Implementations
// ---------------------------------------------------------------------------

static void lcm_set_util_funcs(const LCM_UTIL_FUNCS *util)
{
	memcpy(&lcm_util, util, sizeof(LCM_UTIL_FUNCS));
}


static void lcm_get_params(LCM_PARAMS *params)
{
	memset(params, 0, sizeof(LCM_PARAMS));

	params->type   = LCM_TYPE_DPI;

	params->dpi.clk_pol           = LCM_POLARITY_RISING;
	params->dpi.de_pol            = LCM_POLARITY_RISING;
	params->dpi.vsync_pol         = LCM_POLARITY_RISING;
	params->dpi.hsync_pol         = LCM_POLARITY_RISING;

	if (strcmp(CONFIG_ARCH_MTK_PROJECT, "gxv3370") == 0) {
		params->width  = 1024;
		params->height = 600;
		params->dpi.width = 1024;
		params->dpi.height = 600;

		params->dpi.PLL_CLOCK = 52;

		params->dpi.hsync_pulse_width = 3;
		params->dpi.hsync_back_porch  = 160;
		params->dpi.hsync_front_porch = 160;
		params->dpi.vsync_pulse_width = 3;
		params->dpi.vsync_back_porch  = 23;
		params->dpi.vsync_front_porch = 12;
	} else if (strcmp(CONFIG_ARCH_MTK_PROJECT, "gxv3340") == 0) {
		params->width  = 480;
		params->height = 272;
		params->dpi.width = 480;
		params->dpi.height = 272;

		params->dpi.PLL_CLOCK = 9;

		params->dpi.hsync_pulse_width = 8;
		params->dpi.hsync_back_porch  = 10;
		params->dpi.hsync_front_porch = 10;
		params->dpi.vsync_pulse_width = 3;
		params->dpi.vsync_back_porch  = 23;
		params->dpi.vsync_front_porch = 12;
	}

	params->dpi.lvds_tx_en = 1;
	params->dpi.format = LCM_DPI_FORMAT_RGB888;
	params->dpi.rgb_order = LCM_COLOR_ORDER_RGB;

}


static void lcm_init(void)
{
}


static void lcm_suspend(void)
{
}


static void lcm_resume(void)
{
}


LCM_DRIVER lvds_wsvga_lcm_drv =
{
	.name			= "lvds_wsvga",
	.set_util_funcs = lcm_set_util_funcs,
	.get_params     = lcm_get_params,
	.init           = lcm_init,
	.suspend        = lcm_suspend,
	.resume         = lcm_resume,
};
