/*
 * Copyright (C) 2016 MediaTek Inc.

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 */

#ifndef __MT6336_IRQ_H
#define __MT6336_IRQ_H

#define CHR_INT_WIDTH 8

#define CHR_S_INT_GEN(_name)	\
	{	\
		.name =  #_name,	\
	}

#define CHR_M_INTS_GEN(adr, raw_adr, enA, maskA, interrupt)	\
	{	\
		.address =  adr,	\
		.raw_address = raw_adr,	\
		.en =  enA,	\
		.set =  (enA) + 1,	\
		.clear =  (enA) + 2,	\
		.mask =  maskA,	\
		.mask_set =	(maskA) + 1, \
		.mask_clear =  (maskA) + 2,	\
		.interrupts = interrupt,	\
	}

#define MT6336_CON_BIT(flagname)	\
	({	\
		const MT6336_PMU_FLAG_TABLE_ENTRY *pFlag = &mt6336_pmu_flags_table[flagname]; \
		pFlag->mask << pFlag->shift;	\
	})

struct chr_interrupt_bit {
	const char *name;
	void (*callback)(void);
	unsigned int times;
};

struct chr_interrupts {
	unsigned int address;
	unsigned int raw_address;
	unsigned int en;
	unsigned int set;
	unsigned int clear;
	unsigned int mask;
	unsigned int mask_set;
	unsigned int mask_clear;
	struct chr_interrupt_bit *interrupts;
};

typedef enum {
#ifdef MT6336_E1
	MT6336_INT_CHR_VBUS_PLUGIN,
	MT6336_INT_CHR_VBUS_PLUGOUT,
	MT6336_INT_STATE_BUCK_BACKGROUND,
	MT6336_INT_STATE_BUCK_EOC,
	MT6336_INT_STATE_BUCK_PRECC0,
	MT6336_INT_STATE_BUCK_PRECC1,
	MT6336_INT_STATE_BUCK_FASTCC,
	MT6336_INT_CHR_WEAKBUS,
	MT6336_INT_CHR_SYS_OVP,
	MT6336_INT_CHR_BAT_OVP,
	MT6336_INT_CHR_VBUS_OVP,
	MT6336_INT_CHR_VBUS_UVLO,
	MT6336_INT_CHR_ICHR_ITERM,
	MT6336_INT_CHIP_TEMP_OVERHEAT,
	MT6336_INT_CHIP_MBATPP_DIS_OC_DIG,
	MT6336_INT_OTG_BVALID,
	MT6336_INT_OTG_VM_UVLO,
	MT6336_INT_OTG_VM_OVP,
	MT6336_INT_OTG_VBAT_UVLO,
	MT6336_INT_OTG_VM_OLP,
	MT6336_INT_FLASH_VFLA_UVLO,
	MT6336_INT_FLASH_VFLA_OVP,
	MT6336_INT_LED1_SHORT,
	MT6336_INT_LED1_OPEN,
	MT6336_INT_LED2_SHORT,
	MT6336_INT_LED2_OPEN,
	MT6336_INT_FLASH_TIMEOUT,
	MT6336_INT_TORCH_TIMEOUT,
	MT6336_INT_DD_VBUS_IN_VALID,
	MT6336_INT_WDT_TIMEOUT,
	MT6336_INT_SAFETY_TIMEOUT,
	MT6336_INT_CHR_AICC_DONE,
	MT6336_INT_ADC_TEMP_HT,
	MT6336_INT_ADC_TEMP_LT,
	MT6336_INT_ADC_JEITA_HOT,
	MT6336_INT_ADC_JEITA_WARM,
	MT6336_INT_ADC_JEITA_COOL,
	MT6336_INT_ADC_JEITA_COLD,
	MT6336_INT_VBUS_SOFT_OVP_H,
	MT6336_INT_VBUS_SOFT_OVP_L,
	MT6336_INT_CHR_BAT_RECHG,
	MT6336_INT_BAT_TEMP_H,
	MT6336_INT_BAT_TEMP_L,
	MT6336_INT_TYPE_C_L_MIN,
	MT6336_INT_TYPE_C_L_MAX,
	MT6336_INT_TYPE_C_H_MIN,
	MT6336_INT_TYPE_C_H_MAX,
	MT6336_INT_TYPE_C_CC_IRQ,
	MT6336_INT_TYPE_C_PD_IRQ,
	MT6336_INT_DD_PE_STATUS,
	MT6336_INT_BC12_V2P7_TIMEOUT,
	MT6336_INT_BC12_V3P2_TIMEOUT,
	MT6336_INT_DD_BC12_STATUS,
	MT6336_INT_DD_SWCHR_TOP_RST_SW,
	MT6336_INT_DD_SWCHR_TOP_RST_GLOBAL,
	MT6336_INT_DD_SWCHR_TOP_RST_LONG_PRESS,
	MT6336_INT_DD_SWCHR_TOP_RST_WDT,
	MT6336_INT_DD_SWCHR_PLUGOUT_PULSEB_RISING,
	MT6336_INT_DD_SWCHR_PLUGOUT_PULSEB_LEVEL,
	MT6336_INT_DD_SWCHR_PLUGIN_PULSEB,
	MT6336_INT_DD_SWCHR_TOP_RST_SHIP,
	MT6336_INT_DD_SWCHR_TOP_RST_BAT_OC,
	MT6336_INT_DD_SWCHR_TOP_RST_BAT_DEAD,
	MT6336_INT_DD_SWCHR_BUCK_MODE,
	MT6336_INT_DD_SWCHR_LOWQ_MODE,
	MT6336_INT_DD_SWCHR_SHIP_MODE,
	MT6336_INT_DD_SWCHR_BAT_OC_MODE,
	MT6336_INT_DD_SWCHR_BAT_DEAD_MODE,
	MT6336_INT_DD_SWCHR_RST_SW_MODE,
	MT6336_INT_DD_SWCHR_RST_GLOBAL_MODE,
	MT6336_INT_DD_SWCHR_RST_WDT_MODE,
	MT6336_INT_DD_SWCHR_RST_LONG_PRESS_MODE,
	MT6336_INT_DD_SWCHR_CHR_SUSPEND_STATE,
	MT6336_INT_DD_SWCHR_BUCK_PROTECT_STATE,
	NO_USE_9_2,
	NO_USE_9_3,
	NO_USE_9_4,
	NO_USE_9_5,
	NO_USE_9_6,
	NO_USE_9_7,
#else
	MT6336_INT_CHR_VBUS_PLUGIN,
	MT6336_INT_CHR_VBUS_PLUGOUT,
	MT6336_INT_STATE_BUCK_BACKGROUND,
	MT6336_INT_STATE_BUCK_EOC,
	MT6336_INT_STATE_BUCK_PRECC0,
	MT6336_INT_STATE_BUCK_PRECC1,
	MT6336_INT_STATE_BUCK_FASTCC,
	MT6336_INT_CHR_WEAKBUS,
	MT6336_INT_CHR_SYS_OVP,
	MT6336_INT_CHR_BAT_OVP,
	MT6336_INT_CHR_VBUS_OVP,
	MT6336_INT_CHR_VBUS_UVLO,
	MT6336_INT_CHR_ICHR_ITERM,
	MT6336_INT_CHIP_TEMP_OVERHEAT,
	MT6336_INT_CHIP_MBATPP_DIS_OC_DIG,
	MT6336_INT_OTG_BVALID,
	MT6336_INT_OTG_VM_UVLO,
	MT6336_INT_OTG_VM_OVP,
	MT6336_INT_OTG_VBAT_UVLO,
	MT6336_INT_OTG_VM_OLP,
	MT6336_INT_FLASH_VFLA_UVLO,
	MT6336_INT_FLASH_VFLA_OVP,
	MT6336_INT_LED1_SHORT,
	MT6336_INT_LED1_OPEN,
	MT6336_INT_LED2_SHORT,
	MT6336_INT_LED2_OPEN,
	MT6336_INT_FLASH_TIMEOUT,
	MT6336_INT_TORCH_TIMEOUT,
	MT6336_INT_DD_VBUS_IN_VALID,
	MT6336_INT_WDT_TIMEOUT,
	MT6336_INT_SAFETY_TIMEOUT,
	MT6336_INT_CHR_AICC_DONE,
	MT6336_INT_ADC_TEMP_HT,
	MT6336_INT_ADC_TEMP_LT,
	MT6336_INT_ADC_JEITA_HOT,
	MT6336_INT_ADC_JEITA_WARM,
	MT6336_INT_ADC_JEITA_COOL,
	MT6336_INT_ADC_JEITA_COLD,
	MT6336_INT_VBUS_SOFT_OVP_H,
	MT6336_INT_VBUS_SOFT_OVP_L,
	MT6336_INT_TYPE_C_L_MIN,
	MT6336_INT_TYPE_C_L_MAX,
	MT6336_INT_TYPE_C_H_MIN,
	MT6336_INT_TYPE_C_H_MAX,
	MT6336_INT_TYPE_C_CC_IRQ,
	MT6336_INT_TYPE_C_PD_IRQ,
	MT6336_INT_CHR_BAT_RECHG,
	MT6336_INT_DD_PE_STATUS,
	MT6336_INT_BAT_TEMP_H,
	MT6336_INT_BAT_TEMP_L,
	MT6336_INT_BC12_V2P7_TIMEOUT,
	MT6336_INT_BC12_V3P2_TIMEOUT,
	MT6336_INT_DD_BC12_STATUS,
	MT6336_INT_DD_SWCHR_TOP_RST_SW,
	MT6336_INT_DD_SWCHR_TOP_RST_GLOBAL,
	MT6336_INT_DD_SWCHR_TOP_RST_LONG_PRESS,
	MT6336_INT_DD_SWCHR_TOP_RST_WDT,
	MT6336_INT_DD_SWCHR_PLUGOUT_PULSEB_RISING,
	MT6336_INT_DD_SWCHR_PLUGIN_PULSEB,
	MT6336_INT_DD_SWCHR_TOP_RST_SHIP,
	MT6336_INT_DD_SWCHR_TOP_RST_BAT_OC,
	MT6336_INT_DD_SWCHR_TOP_RST_BAT_DEAD,
	MT6336_INT_DD_SWCHR_BUCK_MODE,
	MT6336_INT_DD_SWCHR_LOWQ_MODE,
	MT6336_INT_DD_SWCHR_SHIP_MODE,
	MT6336_INT_DD_SWCHR_BAT_OC_MODE,
	MT6336_INT_DD_SWCHR_BAT_DEAD_MODE,
	MT6336_INT_DD_SWCHR_RST_SW_MODE,
	MT6336_INT_DD_SWCHR_RST_GLOBAL_MODE,
	MT6336_INT_DD_SWCHR_RST_WDT_MODE,
	MT6336_INT_DD_SWCHR_RST_LONG_PRESS_MODE,
	MT6336_INT_DD_SWCHR_CHR_SUSPEND_STATE,
	MT6336_INT_DD_SWCHR_BUCK_PROTECT_STATE,
	MT6336_INT_DD_PAM_MODE_DB_RISING,
	MT6336_INT_DD_PAM_MODE_DB_FALLING,
	MT6336_INT_DD_QI_THR_MODE_DB_RISING,
	MT6336_INT_DD_QI_THR_MODE_DB_FALLING,
	MT6336_INT_DD_CV_MODE_DB_RISING,
	MT6336_INT_DD_CV_MODE_DB_FALLING,
	MT6336_INT_DD_QI_ICL_MODE_DB_RISING,
	MT6336_INT_DD_QI_ICL_MODE_DB_FALLING,
	MT6336_INT_DD_QI_ICC_MODE_DB_RISING,
	MT6336_INT_DD_QI_ICC_MODE_DB_FALLING,
	MT6336_INT_DD_1MS_TIMER_FINISH,
	MT6336_NO_USE_10_4,
	MT6336_NO_USE_10_5,
	MT6336_NO_USE_10_6,
	MT6336_NO_USE_10_7,
#endif
} MT6336_IRQ_ENUM;

/* mt6336 irq extern variable */
#if !defined CONFIG_HAS_WAKELOCKS
extern struct wakeup_source mt6336Thread_lock;
#else
extern struct wake_lock mt6336Thread_lock;
#endif
extern struct chr_interrupts mt6336_interrupts[];
/* mt6336 irq extern functions */
extern void MT6336_EINT_SETTING(void);

extern void mt6336_enable_interrupt(MT6336_IRQ_ENUM intNo, char *str);
extern void mt6336_disable_interrupt(MT6336_IRQ_ENUM intNo, char *str);
extern void mt6336_mask_interrupt(MT6336_IRQ_ENUM intNo, char *str);
extern void mt6336_unmask_interrupt(MT6336_IRQ_ENUM intNo, char *str);
extern void mt6336_register_interrupt_callback(MT6336_IRQ_ENUM intNo, void (EINT_FUNC_PTR) (void));

#endif /*--__MT6336_IRQ_H--*/
