#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/rslib.h>
#include <linux/moduleparam.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/types.h>
#include <linux/clk.h>
#include <linux/interrupt.h>
#include <linux/semaphore.h>
#include <linux/io.h>
#include <linux/slab.h>
#include <linux/completion.h>
#include <linux/uaccess.h>
#include <linux/proc_fs.h>
#include <linux/vmalloc.h>
#include <linux/module.h>

#include <mach/gs_type.h>
#include "mmc_rw.h"

#define BCD_SECTOR_OFFSET (BCB_SECTOR_OFFSET)
#define BCD_SECTOR_NUM (64)
#define aligned_sector(x) (x?(((x-1)>>9) + 1):0)

struct bootloader_message {
	char command[32];
	char status[32];
	char recovery[1024];
};

static ssize_t bcd_proc_write_mmc(struct file *file,
	const char __user *buffer, size_t count, loff_t *pos)
{
	int rval = 0;
	struct bootloader_message *pmsg;
	int sec_n = aligned_sector(sizeof(struct bootloader_message));

	if (count > sizeof(struct bootloader_message))
		return -EINVAL;

	pmsg = (struct bootloader_message *)vmalloc(sec_n*512);
	if (!pmsg)
		return -ENOMEM;

	memset(pmsg, 0, sec_n*512);

	if (copy_from_user(pmsg, buffer, count)) {
		vfree(pmsg);
		return -EFAULT;
	}

	rval = emmc_write(BCD_SECTOR_OFFSET, sec_n, (char *)pmsg, sec_n*512);
	if (rval < 0) {
		vfree(pmsg);
		return -EIO;
	}

	vfree(pmsg);
	return count;
}

static int bcd_proc_show_mmc(struct seq_file *m, void *v)
{
	int len, rval, sec_n, i = 0;
	struct bootloader_message *pmsg = NULL;
	char *data = NULL;

	sec_n = aligned_sector(sizeof(struct bootloader_message));
	if (sec_n > BCD_SECTOR_NUM) {
		return -EINVAL;
	}

	pmsg = (struct bootloader_message *)vmalloc(sec_n*512);
	if (!pmsg)
		return -ENOMEM;

	memset(pmsg, 0, sec_n*512);
	data = (char *)pmsg;

	rval = emmc_read(BCD_SECTOR_OFFSET, sec_n, (char *)pmsg, sec_n*512);
	if (rval > 0) {
		vfree(pmsg);
		return -EIO;
	}

	len = sizeof(struct bootloader_message);

	for (i = 0; i < len; i++) {
		seq_printf(m, "%c", *data);
		data++;
	}

	vfree(pmsg);
	return 0;
}

static int bcd_proc_open_mmc(struct inode *inode, struct file *file)
{
	return single_open(file, bcd_proc_show_mmc, NULL);
}

static const struct file_operations bcd_proc_mmc_fops = {
	.open  = bcd_proc_open_mmc,
	.read  = seq_read,
	.write  = bcd_proc_write_mmc,
	.release = single_release,
};

int create_bcd_proc_emmc(void)
{
	struct proc_dir_entry *ent;

	ent = proc_create("bootloader_bcd", S_IFREG | S_IRUGO,
			NULL, &bcd_proc_mmc_fops);
	if (!ent) {
		printk("can not create proc node\n");
		return -1;
	}

	return 0;
}

static int __init bcd_init(void)
{
	int storage = GS_STORAGE_EMMC;
	int ret = 0;

	switch (storage) {
	case GS_STORAGE_EMMC:
#ifdef CONFIG_GS_STORAGE_EMMC
		printk("BCD emmc storage\n");
		ret = create_bcd_proc_emmc();
#endif
		break;
	case GS_STORAGE_FLASH:
	default:
		printk("BCD storage type not supported\n");
		ret = -EINVAL;
		break;
	}

	return ret;
}

static void __exit bcd_exit(void)
{
}

module_init(bcd_init);
module_exit(bcd_exit);
MODULE_LICENSE("GPL");
